/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.w3c.dom.Document;

import sitool.components.EnumWrapper;
import sitool.components.Version;
import sitool.i18n.Language;
import sitool.resources.ImportExportPane;
import sitool.table.AccountTableModel;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.ImageTableCellRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.ProductionBonusTableCellEditor;
import sitool.table.SpecializationTableCellEditor;
import sitool.table.SpinnerCellEditor;
import sitool.table.TooltipTable;


/**
 *
 * @author graf
 *
 */
public class AccountBox extends JDialog implements ActionListener, TableModelListener, ImportExportPane
{
    private static final long serialVersionUID = -5828822386166972579L;

    private static SimpleDateFormat fileFormat = new SimpleDateFormat( "yyyyMMddHHmm" );
    private static final String extension = ".saz";

    private MainFrame mainFrame = null;
    private ConfigData configData;

    private JTextField accountName = new JTextField();
    private JCheckBox defaultAccount = new JCheckBox();
    private JComboBox<EnumWrapper<Language>> accountLang = new JComboBox<>();
    private JComboBox<EnumWrapper<Version>> accountVersion = new JComboBox<>();
    private JSpinner speed = new JSpinner();

    private AccountTableModel accounts;
    private JTable accountTable;
    private List<Account> accounts2Save = new ArrayList<Account>();

	public AccountBox( MainFrame parent )
    {
        super( parent );
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "Account.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 850, 340 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );

        pack();
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JPanel northPanel = new JPanel( new GridBagLayout() );
        GridBagConstraints fieldConstraints = new GridBagConstraints();
        fieldConstraints.gridheight = 1;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.fill = GridBagConstraints.BOTH;
        fieldConstraints.anchor = GridBagConstraints.CENTER;
        fieldConstraints.weighty = 1.0;
        fieldConstraints.weightx = 1.0;
        fieldConstraints.insets = new Insets( 5, 5, 5, 5 );

        fieldConstraints.gridx = 0;
        northPanel.add( new JLabel( configData.getI18nText( "Account.name" ) ), fieldConstraints );
        fieldConstraints.gridx = 4;
        northPanel.add( new JLabel( configData.getI18nText( "Account.isdefault" ) ), fieldConstraints );
        fieldConstraints.gridy = 1;
        fieldConstraints.gridx = 0;
        fieldConstraints.gridwidth = 4;
        northPanel.add( accountName, fieldConstraints );
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        northPanel.add( defaultAccount, fieldConstraints );

        fieldConstraints.gridy = 2;
        fieldConstraints.gridx = 0;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 0.0;
        northPanel.add( new JLabel( configData.getI18nText( "Account.language" ) ), fieldConstraints );
        for( Language l : Language.values() )
        {
            accountLang.addItem( new EnumWrapper<Language>( configData, l ) );
        }
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 1.0;
        northPanel.add( accountLang, fieldConstraints );
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 0.0;
        northPanel.add( new JLabel( configData.getI18nText( "Account.speed" ) ), fieldConstraints );
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 1.0;
        speed.setModel( new SpinnerNumberModel( 1.0f, 1.0f, Integer.MAX_VALUE, 0.5f ) );
        northPanel.add( speed, fieldConstraints );
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 0.0;
        northPanel.add( new JLabel( configData.getI18nText( "Account.version" ) ), fieldConstraints );
        fieldConstraints.gridx = GridBagConstraints.RELATIVE;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.weightx = 1.0;
        northPanel.add( accountVersion, fieldConstraints );

        fieldConstraints.gridy = 3;
        fieldConstraints.gridx = 1;
        fieldConstraints.gridwidth = 2;
        JButton button = new JButton( configData.getI18nText( "Account.create" ) );
        button.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                actionAddAccount();
            }

        });
        northPanel.add( button, fieldConstraints );
        center.add(northPanel, BorderLayout.NORTH );
        getRootPane().setDefaultButton( button );

        for( Version v : Version.values() )
        {
            accountVersion.addItem( new EnumWrapper<Version>( configData, v ) );
        }

        accounts = new AccountTableModel( mainFrame );
        accounts.addTableModelListener( this );
        final String pattern = ConfigData.speedFormat.toPattern();
        final TableCellEditor col0Editor = new DefaultCellEditor( new JTextField() );
        final TableCellRenderer checkBoxRenderer = new CheckBoxTableCellRenderer();
        final NumericSpinnerTableCellRenderer speedRenderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0.0f, 0.0f, Integer.MAX_VALUE, 0.5f ) );
        final NumericSpinnerTableCellRenderer speedUpRenderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0.0f, 0.0f, Integer.MAX_VALUE, 0.5f ) );
        speedRenderer.setEditor(new JSpinner.NumberEditor(speedRenderer.getSpinnerComponent(), pattern));
        speedUpRenderer.setEditor(new JSpinner.NumberEditor(speedUpRenderer.getSpinnerComponent(), pattern));
        JCheckBox checkBox = new JCheckBox();
        checkBox.setHorizontalAlignment( SwingConstants.CENTER );
        final TableCellEditor checkBoxEditor  = new DefaultCellEditor( checkBox );
        JComboBox<EnumWrapper<Language>> langBox = new JComboBox<>();
        for( Language l : Language.values() ) {
            langBox.addItem( new EnumWrapper<Language>( configData, l ) );
        }
        final TableCellEditor col2Editor = new DefaultCellEditor( langBox );
        final SpinnerCellEditor speedEditor = new SpinnerCellEditor( new SpinnerNumberModel( 1.0, 1.0, Integer.MAX_VALUE, 0.5f ) );
        final SpinnerCellEditor speedUpEditor = new SpinnerCellEditor( new SpinnerNumberModel( 1.0, 1.0, Integer.MAX_VALUE, 0.5f ) );
        final AlternateLineTableCellRenderer specializationRenderer = new AlternateLineTableCellRenderer();
        specializationRenderer.setHorizontalAlignment(SwingConstants.CENTER);
        final AlternateLineTableCellRenderer productionBonusRenderer = new AlternateLineTableCellRenderer();
        productionBonusRenderer.setHorizontalAlignment(SwingConstants.CENTER);
        final SpecializationTableCellEditor specializationEditor = new SpecializationTableCellEditor(configData, this);
        final ProductionBonusTableCellEditor productionBonusEditor = new ProductionBonusTableCellEditor(configData, this);
        speedEditor.setEditor(new JSpinner.NumberEditor(speedEditor.getSpinnerComponent(), pattern));
        speedUpEditor.setEditor(new JSpinner.NumberEditor(speedUpEditor.getSpinnerComponent(), pattern));
        Dimension spectralSize = new Dimension(15,15);
        Icon spectralOn = ConfigData.getIcon("/spectral_on.png", spectralSize);
        Icon spectralOff = ConfigData.getIcon("/spectral_off.png", spectralSize);
        final TableCellRenderer spectralRenderer = new CheckBoxTableCellRenderer(spectralOn, spectralOff);
        JCheckBox spectralBox = new JCheckBox();
        spectralBox.setIcon(spectralOff);
        spectralBox.setSelectedIcon(spectralOn);
        final TableCellEditor spectralEditor = new DefaultCellEditor( spectralBox );
        JComboBox<EnumWrapper<Version>> versionBox = new JComboBox<>();
        for( Version v : Version.values() ) {
            versionBox.addItem( new EnumWrapper<Version>( configData, v ) );
        }
        final TableCellEditor versionEditor = new DefaultCellEditor( versionBox );
        final ImageTableCellRenderer ic = new ImageTableCellRenderer();
        accountTable = new TooltipTable( accounts ) {
            private static final long serialVersionUID = -7124731757030093187L;

            public TableCellEditor getCellEditor( int row, int column )
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn =  convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case AccountTableModel.NAME:
                        rv = col0Editor;
                        break;
                    case AccountTableModel.DEFAULT:
                    case AccountTableModel.OUTPOST:
                    case AccountTableModel.IMPORT_PLANETS:
                    case AccountTableModel.IMPORT_SPY_REPORTS:
                        rv = checkBoxEditor;
                        break;
                    case AccountTableModel.LANGUAGE:
                        rv = col2Editor;
                        break;
                    case AccountTableModel.SPEED:
                    case AccountTableModel.FLEETSPEED:
                        rv = speedEditor;
                        break;
                    case AccountTableModel.SPEEDUP:
                        rv = speedUpEditor;
                        break;
                    case AccountTableModel.SPECIALIZATION:
                        rv = specializationEditor;
                        break;
                    case AccountTableModel.PRODUCTIONBONUS:
                        rv = productionBonusEditor;
                        break;
                    case AccountTableModel.SPECTRAL:
                        rv = spectralEditor;
                        break;
                    case AccountTableModel.VERSION:
                        rv = versionEditor;
                        break;
                    default:
                        rv = super.getCellEditor( row, column );
                }
                return rv;
            }

            public TableCellRenderer getCellRenderer( int row, int column )
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn =  convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case AccountTableModel.DEFAULT:
                    case AccountTableModel.OUTPOST:
                    case AccountTableModel.IMPORT_PLANETS:
                    case AccountTableModel.IMPORT_SPY_REPORTS:
                        rv = checkBoxRenderer;
                        break;
                    case AccountTableModel.SPEED:
                    case AccountTableModel.FLEETSPEED:
                        rv = speedRenderer;
                        break;
                    case AccountTableModel.SPEEDUP:
                        rv = speedUpRenderer;
                        break;
                    case AccountTableModel.SPECIALIZATION:
                        rv = specializationRenderer;
                        break;
                    case AccountTableModel.PRODUCTIONBONUS:
                        rv = productionBonusRenderer;
                        break;
                    case AccountTableModel.SPECTRAL:
                        rv = spectralRenderer;
                        break;
                    case AccountTableModel.UP:
                    case AccountTableModel.DOWN:
                        rv = ic;
                        break;
	                default:
	                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }
        };
        accountTable.setDefaultRenderer( Object.class, new AlternateLineTableCellRenderer() );
        accountTable.addMouseListener( accounts );

        JScrollPane scrollPane = new JScrollPane( accountTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        accountTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        center.add( scrollPane, BorderLayout.CENTER );
        final JButton removeAccount = new JButton( configData.getI18nText( "Account.remove.selected" ) );
        removeAccount.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                int[] rows = accountTable.getSelectedRows();
                accounts.removeRows( rows );
            }

        });
        removeAccount.setEnabled( false );
        final JButton importButton = new JButton( configData.getI18nText( "Import" ) );
        importButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionLoad();
            }

        });
        importButton.setEnabled( true );
        final JButton exportButton = new JButton( configData.getI18nText( "Save" ) );
        exportButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSave();
            }

        });
        exportButton.setEnabled( false );

        JPanel actionPanel = new JPanel( new FlowLayout( FlowLayout.CENTER, 5, 5 ) );
        JPanel buttonPanel = new JPanel( new GridLayout( 3, 1, 4, 4) );
        buttonPanel.add(  importButton );
        buttonPanel.add(  exportButton );
        buttonPanel.add(  removeAccount );
        actionPanel.add( buttonPanel );
        center.add( actionPanel, BorderLayout.EAST );

        accountTable.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    // enable the button when selection is active
                    boolean isSelected = ( e.getFirstIndex() >= 0 );
                    removeAccount.setEnabled( isSelected );
                    exportButton.setEnabled( isSelected );
                }
            }
        });

        int width = 16;
        TableColumn col;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.UP );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = accountTable.getColumnModel().getColumn( AccountTableModel.DOWN );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 30;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.OUTPOST );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = accountTable.getColumnModel().getColumn( AccountTableModel.SPECTRAL );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = accountTable.getColumnModel().getColumn( AccountTableModel.IMPORT_PLANETS );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = accountTable.getColumnModel().getColumn( AccountTableModel.IMPORT_SPY_REPORTS );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 110;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.NAME );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 35;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.DEFAULT );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = accountTable.getColumnModel().getColumn( AccountTableModel.LANGUAGE );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 55;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.SPECIALIZATION );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 95;
        col = accountTable.getColumnModel().getColumn( AccountTableModel.PRODUCTIONBONUS );
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );

        // set initial values
        if( mainFrame != null )
        {
            accounts.setData( mainFrame.getConfigData().getAccounts() );
        }
        defaultAccount.setSelected( !accounts.hasDefault() );
        return center;
    }

    @SuppressWarnings("unchecked")
    protected void actionAddAccount()
    {
        String name = accountName.getText();
        if( name.trim().length() > 0 )
        {
            Account current = new Account( name, defaultAccount.isSelected() );
            current.setLanguage( ( (EnumWrapper<Language>)accountLang.getSelectedItem() ).getBase() );
            current.setVersion( ( (EnumWrapper<Version>)accountVersion.getSelectedItem() ).getBase() );
            current.setSpeedFactor( ((Number)speed.getValue()).doubleValue() );
            current.setFleetSpeed( ((Number)speed.getValue()).doubleValue() );
            if( !accounts.getData().contains( current ) )
            {
                if( defaultAccount.isSelected() )
                {
                    // remove previous selection
                    for( Account account : accounts.getData() )
                    {
                        account.setDefault( false );
                    }
                    accounts.fireTableStructureChanged();
                }
                accounts.addRow( current );
                accountName.setText( "" );
            }
            else
            {
                JOptionPane.showConfirmDialog( this,
                                configData.getI18nText( "Account.already.exists" ),
                                configData.getI18nText( "Warning" ),
                                JOptionPane.DEFAULT_OPTION,
                                JOptionPane.INFORMATION_MESSAGE
                                );
            }
        }
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        int result = JOptionPane.OK_OPTION;
        if( accountName.getText().length() > 0 )
        {
            result = JOptionPane.showConfirmDialog( this,
                                configData.getI18nText( "Account.unsaved.data" ),
                                configData.getI18nText( "Really.quit" ),
                                JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE
                            );

        }

        if( result == JOptionPane.OK_OPTION )
        {
          	configData.setAccounts( accounts.getData() );
            dispose();
        }
    }

    public void tableChanged( TableModelEvent e )
    {
        defaultAccount.setSelected( !accounts.hasDefault() );
    }

    //******************* import / export **********************
    protected void actionLoad()
    {
        Document doc = configData.readData( this );
        if( doc != null )
        {
            List<Account> data = accounts.getData();
            List<Account> newData = configData.readAccountsFromConfig( doc );

            if( newData != null )
            {
                for( Account account : newData )
                {
                    int index = data.indexOf( account );
                    if( index >= 0 )
                    {
                        if( JOptionPane.showConfirmDialog( this,
                                        configData.getI18nText( "Account.already.exists" ),
                                        configData.getI18nText( "confirm.overwrite" ),
                                        JOptionPane.YES_NO_OPTION,
                                        JOptionPane.QUESTION_MESSAGE
                                        ) == JOptionPane.OK_OPTION );
                        {
                            Account old = data.remove( index );
                            // retain old default setting
                            account.setDefault( old.isDefault() );
                            data.add( index, account );
                        }

                    }
                    else
                    {
                        // do not import 'default' setting
                        account.setDefault( false );
                        data.add( account );
                    }
                }
            }
            accounts.setData( data );
        }
    }

    protected void actionSave()
    {
        int[] rows = accountTable.getSelectedRows();
        accounts2Save.addAll( accounts.getRows( rows ) );
        configData.saveData( this );
        accounts2Save.clear();
    }


    public String getExtension()
    {
        return extension;
    }

    public String getFileName()
    {
        StringBuilder sb = new StringBuilder();
        sb.append( fileFormat.format( new Date() ) ).append( ' ' );
        if( accounts2Save != null && accounts2Save.size() > 0 && accounts2Save.size() < accounts.getRowCount() )
        {
            for( int i = 0; i < accounts2Save.size(); i++ )
            {
                if( i > 0 )
                {
                    sb.append( ", " );
                }
                Account account = accounts2Save.get( i );
                sb.append( account.getId() );
            }
        }
        else
        {
            sb.append( configData.getI18nText( "Account.title" ) );
        }
        return sb.toString();
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    public File getCurrentDir()
    {
        return configData.getAccountDir();
    }

    public void setCurrentDir( File currentDir )
    {
        configData.setAccountDir( currentDir );
    }


    public Document getData( Parser xmlParser )
    {
        Document doc = null;
        if( xmlParser != null )
        {
            doc = xmlParser.getDocument();
            configData.addAccounts2Config( doc, accounts2Save );
        }
        return doc;
    }

}
