/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.components.LaFStyle;
import sitool.components.ModulePane;
import sitool.components.OvalButton;
import sitool.components.TotalPointDisplay;
import sitool.components.TotalPointSource;
import sitool.components.Version;
import sitool.datatransfer.DataServer;
import sitool.fleet.FleetPane;
import sitool.planets.CoordinatorPane;
import sitool.planets.PlanetPane;
import sitool.resources.BuildingSummaryPane;
import sitool.resources.ResourceSummaryPane;
import sitool.savecalc.FlightTimePane;
import sitool.savecalc.SaveCalc;
import sitool.statistic.StatisticsPane;
import sitool.technology.ResearchPane;
import sitool.technology.TechTreePane;
import sitool.trade.TradeSummaryPane;

/**
 *
 * @author graf
 *
 */
public class MainFrame extends JFrame implements ComponentListener
{
	private static final long serialVersionUID = -1878874776754387489L;

	protected static final int HELP = 1;
    protected static final int ABOUT = 2;

    private ConfigData configData;

    private JTabbedPane pane;

    private JComboBox<Object> accountBox;
    private TotalPointDisplay display;
    private Map<TotalPointSource.Type, TotalPointSource> totalPointSources = new HashMap<TotalPointSource.Type, TotalPointSource>();

    private JMenu jMenuFile;
    private JMenuItem jMenuFileExit;
    private JMenu jMenuHelp;
    private JMenuItem jMenuHelpHelp;
    private JMenuItem jMenuHelpAbout;

    private JMenu jMenuSkins;
    private JRadioButtonMenuItem jMenuSkinsDefault;
    private JRadioButtonMenuItem jMenuSkinsBlack;
    private JRadioButtonMenuItem jMenuSkinsBlue;
    private JRadioButtonMenuItem jMenuSkinsBlueSteel;
    private JRadioButtonMenuItem jMenuSkinsBlackSteel;
    private JRadioButtonMenuItem jMenuSkinsCreme;
    private JRadioButtonMenuItem jMenuSkinsChallenger;
    private JRadioButtonMenuItem jMenuSkinsDust;
    private JRadioButtonMenuItem jMenuSkinsEmerald;
    private JRadioButtonMenuItem jMenuSkinsGraphite;
    private JRadioButtonMenuItem jMenuSkinsNebula;
    private JRadioButtonMenuItem jMenuSkinsRaven;
    private JRadioButtonMenuItem jMenuSkinsSahara;
    private JRadioButtonMenuItem jMenuSkinsSilver;

    private JLabel labelAccount;
    private JLabel valueSpeed;
    private JButton buttonAccount;
    private OvalButton syncButton;
    private JToggleButton spectralButton;
    private JPanel indicatorPanel;

    private Map<String, Integer> titles = new HashMap<String, Integer>();
    private boolean restart = false;

    private final DataServer dataServer;

    /**
     * @throws HeadlessException
     */
    public MainFrame() throws HeadlessException
    {
        super();
        this.setIconImage( new ImageIcon( this.getClass().getResource( "/symbol.png" ) )
                        .getImage() );
        enableEvents( AWTEvent.WINDOW_EVENT_MASK );

        // read configuration
        configData = new ConfigData();
        configData.readConfigData();

        dataServer = new DataServer(this);

        setTitle( "SI Tool");
        LaFStyle style = configData.getCurrentStyle();
        setLookAndFeel( style.getClassName() );

        jMenuFile = new JMenu();
        jMenuFileExit = new JMenuItem();
        jMenuHelp = new JMenu();
        jMenuHelpHelp = new JMenuItem();
        jMenuHelpAbout = new JMenuItem();

        jMenuSkins = new JMenu();
        jMenuSkinsDefault = new JRadioButtonMenuItem();
        jMenuSkinsBlack = new JRadioButtonMenuItem();
        jMenuSkinsBlue = new JRadioButtonMenuItem();
        jMenuSkinsBlueSteel = new JRadioButtonMenuItem();
        jMenuSkinsBlackSteel = new JRadioButtonMenuItem();
        jMenuSkinsCreme = new JRadioButtonMenuItem();
        jMenuSkinsChallenger = new JRadioButtonMenuItem();
        jMenuSkinsDust = new JRadioButtonMenuItem();
        jMenuSkinsEmerald = new JRadioButtonMenuItem();
        jMenuSkinsGraphite = new JRadioButtonMenuItem();
        jMenuSkinsNebula = new JRadioButtonMenuItem();
        jMenuSkinsRaven = new JRadioButtonMenuItem();
        jMenuSkinsSahara = new JRadioButtonMenuItem();
        jMenuSkinsSilver = new JRadioButtonMenuItem();

        labelAccount = new JLabel();
        buttonAccount = new JButton();

        valueSpeed = new JLabel();

        selectStyle( style );

        JPanel contentPane = ( JPanel )MainFrame.this.getContentPane();
        setJMenuBar( createMenu() );

        JPanel northPanel = new JPanel();
        northPanel.setLayout( new BoxLayout( northPanel, BoxLayout.X_AXIS ) );
        northPanel.setBorder( BorderFactory.createEtchedBorder() );

        JPanel accountPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
        accountPanel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED )  );
        JPanel p = new JPanel();
        p.setLayout( new BoxLayout( p, BoxLayout.X_AXIS ) );
        accountPanel.add( p );
        p.add( labelAccount );
        p.add( Box.createHorizontalStrut( 8 ) );
        accountBox = new JComboBox<>();
        updateAccounts();
        checkStartDataServer();
        accountBox.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionComboBoxChanged( e );
            }

        });

        p.add( accountBox );
        p.add( Box.createHorizontalStrut( 8 ) );
//        p.add( labelSpeed );
//        p.add( Box.createHorizontalStrut( 4 ) );
        p.add( valueSpeed );
        p.add( Box.createHorizontalStrut( 8 ) );
        buttonAccount.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionAccount();
            }
        });
        p.add( buttonAccount );
        northPanel.add( accountPanel );
        northPanel.add( Box.createHorizontalStrut( 1 ) );

        indicatorPanel = new JPanel(new CardLayout(0, 0)) {
            private static final long serialVersionUID = 7819754708556992342L;

            @Override
            public Dimension getPreferredSize() {
                Dimension size = super.getPreferredSize();
                return new Dimension(size.height, size.height);
            }

            @Override
            public Dimension getMaximumSize() {
                return getPreferredSize();
            }

            @Override
            public Dimension getMinimumSize() {
                return getPreferredSize();
            }
        };
        indicatorPanel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED )  );
        final Dimension buttonSize = new Dimension( 40, 40);
        spectralButton = new JCheckBox();
        spectralButton.setBorder(null);
        spectralButton.setIcon(ConfigData.getIcon("/spectral_off.png", buttonSize));
        spectralButton.setSelectedIcon(ConfigData.getIcon("/spectral_on.png", buttonSize));
        spectralButton.setMinimumSize(buttonSize);
        spectralButton.setMaximumSize(buttonSize);
        spectralButton.setPreferredSize(buttonSize);
        spectralButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed( ActionEvent e ) {
                Account account = configData.getActiveAccount();
                if(account != null) {
                    account.setSpectralMode(spectralButton.isSelected());
                    actionUpdatePane();
                }
            }
        });
        indicatorPanel.add(spectralButton, Boolean.toString(false));
        syncButton = new OvalButton();
        syncButton.setArmable( false );
        syncButton.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                Integer index = titles.get( "Coordinator.title" );
                if( index != null && pane.isEnabledAt( index ) )
                {
                    pane.setSelectedIndex( index );
                }
            }
        } );
        indicatorPanel.add(syncButton, Boolean.toString(true));
        northPanel.add( indicatorPanel );
        northPanel.add( Box.createHorizontalStrut( 1 ) );

        display = new TotalPointDisplay( MainFrame.this );
        display.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED )  );
        northPanel.add( Box.createHorizontalGlue() );
        northPanel.add( display );

        contentPane.add( northPanel, BorderLayout.NORTH );

        JPanel centerPanel = createCenterPanel();
        contentPane.add( centerPanel, BorderLayout.CENTER );

        setSize( configData.getSize() );
        addComponentListener( this );
    }

    private void checkStartDataServer() {
        // if at least one account is requesting http data start the server
        boolean startDataServer = false;
        for( Account account : configData.getAccounts()) {
            startDataServer = account.isMayImportPlanetaryOverview() || account.isMayImportSpyReports();
            if(startDataServer)
                break;
        }
        if(startDataServer && !isDataServerRunning()) {
            dataServer.startServer();
        }
        else if(!startDataServer && isDataServerRunning()) {
            dataServer.stopServer();
        }
    }

    protected void selectStyle( LaFStyle style )
    {
        switch( style )
        {
            case black:
                jMenuSkinsBlack.setSelected( true );
                break;
            case blacksteel:
                jMenuSkinsBlackSteel.setSelected( true );
                break;
            case blue:
                jMenuSkinsBlue.setSelected( true );
                break;
            case bluesteel:
                jMenuSkinsBlueSteel.setSelected( true );
                break;
            case creme:
                jMenuSkinsCreme.setSelected( true );
                break;
            case challenger:
                jMenuSkinsChallenger.setSelected( true );
                break;
            case dust:
                jMenuSkinsDust.setSelected( true );
                break;
            case emerald:
                jMenuSkinsEmerald.setSelected( true );
                break;
            case graphite:
                jMenuSkinsGraphite.setSelected( true );
                break;
            case nebula:
                jMenuSkinsNebula.setSelected( true );
                break;
            case raven:
                jMenuSkinsRaven.setSelected( true );
                break;
            case sahara:
                jMenuSkinsSahara.setSelected( true );
                break;
            case silver:
                jMenuSkinsSilver.setSelected( true );
                break;
            default:
                jMenuSkinsDefault.setSelected( true );
                break;
        }
    }

    public ConfigData getConfigData()
    {
        return configData;
    }

    public TotalPointDisplay getDisplay()
    {
        return display;
    }

    public boolean isDataServerRunning() {
        return dataServer.getPort() != 0;
    }

    public void registerTotalPointSource( TotalPointSource.Type type, TotalPointSource source )
    {
        totalPointSources.put( type, source );
        source.addTableModelListener( display );
        display.update();
    }

    public TotalPointSource getTotalPointSource( TotalPointSource.Type type )
    {
        return totalPointSources.get( type );
    }

    protected JMenuBar createMenu()
    {
        JMenuBar jMenuBar = new JMenuBar();
        jMenuFileExit.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK ) );
        jMenuHelpHelp.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F1, 0 ) );
        jMenuHelpAbout.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F1, InputEvent.CTRL_DOWN_MASK ) );

        jMenuFileExit.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                jMenuFileExit_actionPerformed( e );
            }
        } );
        jMenuFile.add( jMenuFileExit );

        ButtonGroup group = new ButtonGroup();
        group.add( jMenuSkinsDefault );
        group.add( jMenuSkinsBlack );
        group.add( jMenuSkinsBlue );
        group.add( jMenuSkinsBlueSteel );
        group.add( jMenuSkinsBlackSteel );
        group.add( jMenuSkinsCreme );
        group.add( jMenuSkinsChallenger );
        group.add( jMenuSkinsDust );
        group.add( jMenuSkinsEmerald );
        group.add( jMenuSkinsGraphite );
        group.add( jMenuSkinsNebula );
        group.add( jMenuSkinsRaven );
        group.add( jMenuSkinsSahara );
        group.add( jMenuSkinsSilver );

        ActionListener l = new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSwitchLaF( e );
            }
        };
        jMenuSkinsDefault.addActionListener( l );
        jMenuSkinsBlack.addActionListener( l );
        jMenuSkinsBlue.addActionListener( l );
        jMenuSkinsBlueSteel.addActionListener( l );
        jMenuSkinsBlackSteel.addActionListener( l );
        jMenuSkinsCreme.addActionListener( l );
        jMenuSkinsChallenger.addActionListener( l );
        jMenuSkinsDust.addActionListener( l );
        jMenuSkinsEmerald.addActionListener( l );
        jMenuSkinsGraphite.addActionListener( l );
        jMenuSkinsNebula.addActionListener( l );
        jMenuSkinsRaven.addActionListener( l );
        jMenuSkinsSahara.addActionListener( l );
        jMenuSkinsSilver.addActionListener( l );
        jMenuSkins.add( jMenuSkinsDefault );
        jMenuSkins.add( new JSeparator() );
        jMenuSkins.add( jMenuSkinsBlack );
        jMenuSkins.add( jMenuSkinsBlackSteel );
        jMenuSkins.add( jMenuSkinsBlue );
        jMenuSkins.add( jMenuSkinsBlueSteel );
        jMenuSkins.add( jMenuSkinsChallenger );
        jMenuSkins.add( jMenuSkinsCreme );
        jMenuSkins.add( jMenuSkinsEmerald );
        jMenuSkins.add( jMenuSkinsGraphite );
        jMenuSkins.add( jMenuSkinsNebula );
        jMenuSkins.add( jMenuSkinsRaven );
        jMenuSkins.add( jMenuSkinsSahara );
        jMenuSkins.add( jMenuSkinsSilver );

        jMenuHelpHelp.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionHelp( HELP );
            }
        } );
        jMenuHelp.add( jMenuHelpHelp );
        jMenuHelpAbout.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionHelp( ABOUT );
            }
        } );
        jMenuHelp.add( jMenuHelpAbout );

        jMenuBar.add( jMenuFile );
        jMenuBar.add( jMenuSkins );
        jMenuBar.add( jMenuHelp );
        return jMenuBar;
    }


    protected void actionSwitchLaF( ActionEvent e )
    {
        LaFStyle style = LaFStyle.defaultLaF;
        if( jMenuSkinsBlueSteel.isSelected() )
        {
            style = LaFStyle.bluesteel;
        }
        else if( jMenuSkinsBlack.isSelected() )
        {
            style = LaFStyle.black;
        }
        else if( jMenuSkinsBlue.isSelected() )
        {
            style = LaFStyle.blue;
        }
        else if( jMenuSkinsBlackSteel.isSelected() )
        {
            style = LaFStyle.blacksteel;
        }
        else if( jMenuSkinsCreme.isSelected() )
        {
            style = LaFStyle.creme;
        }
        else if( jMenuSkinsChallenger.isSelected() )
        {
            style = LaFStyle.challenger;
        }
        else if( jMenuSkinsDust.isSelected() )
        {
            style = LaFStyle.dust;
        }
        else if( jMenuSkinsEmerald.isSelected() )
        {
            style = LaFStyle.emerald;
        }
        else if( jMenuSkinsGraphite.isSelected() )
        {
            style = LaFStyle.graphite;
        }
        else if( jMenuSkinsNebula.isSelected() )
        {
            style = LaFStyle.nebula;
        }
        else if( jMenuSkinsRaven.isSelected() )
        {
            style = LaFStyle.raven;
        }
        else if( jMenuSkinsSahara.isSelected() )
        {
            style = LaFStyle.sahara;
        }
        else if( jMenuSkinsSilver.isSelected() )
        {
            style = LaFStyle.silver;
        }

        if( style == LaFStyle.defaultLaF && configData.getCurrentStyle() != LaFStyle.defaultLaF
         || style != LaFStyle.defaultLaF && configData.getCurrentStyle() == LaFStyle.defaultLaF
          )
        {
            int result = JOptionPane.showConfirmDialog( this,
                            configData.getI18nText( "MainFrame.menu.skins.restart.text" ),
                            configData.getI18nText( "MainFrame.menu.skins.restart" ),
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE
                        );
            if( result == JOptionPane.OK_OPTION )
            {
                configData.setCurrentStyle( style );
                restart = true;
                jMenuFileExit_actionPerformed(null);
            }
            selectStyle( configData.getCurrentStyle() );

        }
        else if( style != configData.getCurrentStyle() )
        {
            configData.setCurrentStyle( style );
            setLookAndFeel( style.getClassName() );
            actionUpdatePane();
        }
    }

    protected void actionAccount()
    {
        JDialog dlg = new AccountBox( this );
        showModalBox( dlg );
        updateAccounts();
        checkStartDataServer();
        display.update();
    }

    protected void actionHelp( int type )
    {
        JDialog dlg;
        switch( type )
        {
            case HELP:
                dlg = new HelpBox( this );
                break;
            default:
                dlg = new AboutBox( this );
                break;
        }
        showModalBox( dlg );
    }

    public void showModalBox( JDialog dialog )
    {
        Dimension dlgSize = dialog.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dialog.setLocation( ( frmSize.width - dlgSize.width ) / 2 + loc.x,
                        ( frmSize.height - dlgSize.height ) / 2 + loc.y );

        dialog.setModal( true );
        dialog.setLocationRelativeTo( this );
        dialog.setVisible( true );
    }

    public void updateAccounts()
    {
        Account current = configData.getActiveAccount();
        accountBox.removeAllItems();
        accountBox.addItem( " ----------- " );
        ArrayList<Account> accounts = new ArrayList<Account>( configData.getAccounts() );
        for( Account account : accounts )
        {
            accountBox.addItem( account );
        }
        setSpeedText(current);
        if( current != null )
        {
            accountBox.setSelectedItem( current );
        }
    }

    public JTabbedPane getTabbedPane()
    {
        return pane;
    }

    public int getTabIndexByTitleKey(String i18nTitleKey ) {
        Integer index = titles.get(i18nTitleKey);
        return index != null ? index.intValue() : -1;
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );
        int index = 0;
        String key;

        pane = new JTabbedPane();
        PlanetPane planetPane = new PlanetPane( this );
        key = "MainFrame.planets";
        pane.add( planetPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        CoordinatorPane coordinatorPane = new CoordinatorPane( this );
        key = "Coordinator.title";
        pane.add( coordinatorPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        SaveCalc saveCalc = new SaveCalc( this );
        key = "MainFrame.savecalc";
        pane.add( saveCalc, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        FlightTimePane flightTime = new FlightTimePane( this );
        key = "MainFrame.flighttime";
        pane.add( flightTime, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        FleetPane fleetPane = new FleetPane( this );
        key = "MainFrame.fleet";
        pane.add( fleetPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        ResearchPane techPane = new ResearchPane( this );
        key = "MainFrame.research";
        pane.add( techPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        BuildingSummaryPane buildingPane = new BuildingSummaryPane( this );
        key = "MainFrame.buildings";
        pane.add( buildingPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        ResourceSummaryPane resourcePane = new ResourceSummaryPane( this );
        key = "MainFrame.resources";
        pane.add( resourcePane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        StatisticsPane statisticsPane = new StatisticsPane( this );
        key = "MainFrame.statistics";
        pane.add( statisticsPane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        TechTreePane techtree = new TechTreePane( this );
        key = "MainFrame.techtree";
        pane.add( techtree, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        TradeSummaryPane tradePane = new TradeSummaryPane( this );
        key = "MainFrame.trade";
        pane.add( tradePane, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        pane.addChangeListener( new ChangeListener()
        {

            public void stateChanged( ChangeEvent e )
            {
                actionUpdatePane();
            }

        });
        // savecalc active by default
        pane.setSelectedIndex( 2 );
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

    public void actionUpdatePane()
    {
        // enable/disable panes
        int selection = pane.getSelectedIndex();
        for( Map.Entry<String, Integer> entry : titles.entrySet() )
        {
            int index = entry.getValue().intValue();
            pane.setTitleAt( index, configData.getI18nText( entry.getKey() ) );
            boolean enabled = true;
            if( entry.getKey() == "Coordinator.title" )
            {
                enabled = configData.getVersion() == Version.v20;
                pane.setEnabledAt( index, enabled );
            }

            if( !enabled && index == selection )
            {
                pane.setSelectedIndex( 2 );
            }
        }

        Component c = pane.getSelectedComponent();
        if( c instanceof ModulePane )
        {
            ((ModulePane)c).update();
            //System.out.println( "update pane " + c.getClass().getSimpleName() );
        }
        updateLabel();
        updateIndicator();
    }

    public void updateIndicator()
    {
        boolean isV2 = (configData.getVersion() == Version.v20);
        ((CardLayout)indicatorPanel.getLayout()).show(indicatorPanel, Boolean.toString(isV2));
        Account account = configData.getActiveAccount();
        if( account != null )
        {
            if(isV2) {
                syncButton.setBackground( ( account.isSync() ) ? Color.green.darker() : Color.red.darker() );
                StringBuilder sb = new StringBuilder();
                if( account.isSync() )
                {
                    sb.append("<html><p align=center style=\"font-size:94%;\"><span style=\"text-decoration:underline;\">");
                    sb.append( account.getGalaxyTicks() ).append( "</span><br><span style=\"text-decoration:overline;\">" ).append( account.getTicks() );
                    sb.append("</span></p></html>");
                    syncButton.setToolTipText( sb.toString() );
                }
                else
                {
                    syncButton.setToolTipText( null );
                }
                syncButton.setText( sb.toString() );
            }
            else {
                spectralButton.setSelected(account.isSpectralMode());
            }
        }
        spectralButton.setEnabled(configData.getVersion() == Version.genesis);
    }

    private void updateLabel()
    {
        jMenuFile.setText( configData.getI18nText( "MainFrame.menu.file" ) );
        jMenuFile.setMnemonic( configData.getI18nText( "MainFrame.menu.file.mnemonic" ).charAt( 0 ) );
        jMenuFileExit.setText( configData.getI18nText( "MainFrame.menu.file.exit" ) );
        jMenuFileExit.setMnemonic( configData.getI18nText( "MainFrame.menu.file.exit.mnemonic" ).charAt( 0 ) );
        jMenuHelp.setText( configData.getI18nText( "MainFrame.menu.help" ) );
        jMenuHelp.setMnemonic( configData.getI18nText( "MainFrame.menu.help.mnemonic" ).charAt( 0 ) );
        jMenuHelpHelp.setText( configData.getI18nText( "MainFrame.menu.help.help" ) );
        jMenuHelpHelp.setMnemonic( configData.getI18nText( "MainFrame.menu.help.help.mnemonic" ).charAt( 0 ) );
        jMenuHelpAbout.setText( configData.getI18nText( "MainFrame.menu.help.info" ) );
        jMenuHelpAbout.setMnemonic( configData.getI18nText( "MainFrame.menu.help.info.mnemonic" ).charAt( 0 ) );

        jMenuSkins.setText( configData.getI18nText( "MainFrame.menu.skins" ) );
        jMenuSkins.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.mnemonic" ).charAt( 0 ) );
        jMenuSkinsDefault.setText( configData.getI18nText( "MainFrame.menu.skins.default" ) );
        jMenuSkinsDefault.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.default.mnemonic" ).charAt( 0 ) );
        jMenuSkinsBlack.setText( configData.getI18nText( "MainFrame.menu.skins.black" ) );
        jMenuSkinsBlack.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.black.mnemonic" ).charAt( 0 ) );
        jMenuSkinsBlue.setText( configData.getI18nText( "MainFrame.menu.skins.blue" ) );
        jMenuSkinsBlue.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.blue.mnemonic" ).charAt( 0 ) );
        jMenuSkinsBlueSteel.setText( configData.getI18nText( "MainFrame.menu.skins.bluesteel" ) );
        jMenuSkinsBlueSteel.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.bluesteel.mnemonic" ).charAt( 0 ) );
        jMenuSkinsBlackSteel.setText( configData.getI18nText( "MainFrame.menu.skins.blacksteel" ) );
        jMenuSkinsBlackSteel.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.blacksteel.mnemonic" ).charAt( 0 ) );
        jMenuSkinsCreme.setText( configData.getI18nText( "MainFrame.menu.skins.creme" ) );
        jMenuSkinsCreme.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.creme.mnemonic" ).charAt( 0 ) );
        jMenuSkinsChallenger.setText( configData.getI18nText( "MainFrame.menu.skins.challenger" ) );
        jMenuSkinsChallenger.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.challenger.mnemonic" ).charAt( 0 ) );
        jMenuSkinsDust.setText( configData.getI18nText( "MainFrame.menu.skins.dust" ) );
        jMenuSkinsDust.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.dust.mnemonic" ).charAt( 0 ) );
        jMenuSkinsEmerald.setText( configData.getI18nText( "MainFrame.menu.skins.emerald" ) );
        jMenuSkinsEmerald.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.emerald.mnemonic" ).charAt( 0 ) );
        jMenuSkinsGraphite.setText( configData.getI18nText( "MainFrame.menu.skins.graphite" ) );
        jMenuSkinsGraphite.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.graphite.mnemonic" ).charAt( 0 ) );
        jMenuSkinsNebula.setText( configData.getI18nText( "MainFrame.menu.skins.nebula" ) );
        jMenuSkinsNebula.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.nebula.mnemonic" ).charAt( 0 ) );
        jMenuSkinsRaven.setText( configData.getI18nText( "MainFrame.menu.skins.raven" ) );
        jMenuSkinsRaven.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.raven.mnemonic" ).charAt( 0 ) );
        jMenuSkinsSahara.setText( configData.getI18nText( "MainFrame.menu.skins.sahara" ) );
        jMenuSkinsSahara.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.sahara.mnemonic" ).charAt( 0 ) );
        jMenuSkinsSilver.setText( configData.getI18nText( "MainFrame.menu.skins.silver" ) );
        jMenuSkinsSilver.setMnemonic( configData.getI18nText( "MainFrame.menu.skins.silver.mnemonic" ).charAt( 0 ) );


        labelAccount.setText( configData.getI18nText( "MainFrame.account" ) );
        //labelSpeed.setText( configData.getI18nText( "Account.speed" ) );
        Account current = configData.getActiveAccount();
        setSpeedText(current);
        buttonAccount.setText( configData.getI18nText( "MainFrame.accounts" ) );
    }

    private void setLookAndFeel( String name )
    {
        try
        {
            UIManager.setLookAndFeel( name );
            // make controls uniformly non-bold
            String key;
            Font font;
            UIDefaults uiDefaults = UIManager.getDefaults();
            key = "Label.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
            key = "Button.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
            key = "RadioButton.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
            key = "CheckBox.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
            key = "ComboBox.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
            key = "Spinner.font";
            font = ( Font )uiDefaults.get( key );
            uiDefaults.put( key, font.deriveFont( Font.PLAIN ) );
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
        for (Window window : Window.getWindows())
        {
            SwingUtilities.updateComponentTreeUI(window);
        }
        setSize( configData.getSize() );
    }


    private void setSpeedText(Account current)
    {
        if( current != null )
        {
            StringBuilder sb = new StringBuilder();
            sb.append(ConfigData.speedFormat.format(current.getSpeedFactor())).append('x');
            sb.append("/").append(ConfigData.speedFormat.format(current.getFleetSpeed())).append('x');
            sb.append("/").append(ConfigData.speedFormat.format(current.getSpeedUp())).append('x');
            valueSpeed.setText( sb.toString() );
        }
        else
        {
            valueSpeed.setText( "1x" );
        }
    }


    /**
     * @param e
     */
    protected void actionComboBoxChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            if( e.getItem() instanceof Account )
            {
                Account account = ( Account )e.getItem();
                configData.setActiveAccount( account );
                configData.adjustCoordinates();
            }
            else
            {
                configData.setActiveAccount( null );
            }
            display.update();
            actionUpdatePane();
        }
    }

    // File | Exit action performed
    /***************************************************************************
     * Description of the Method
     *
     * @param e
     *            Description of the Parameter
     **************************************************************************/
    public void jMenuFileExit_actionPerformed( ActionEvent e )
    {
        // write configuration
        configData.saveConfigData();
        dataServer.stopServer();
        Main.finish( restart );
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     *
     * @param e
     *            Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        super.processWindowEvent( e );
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            jMenuFileExit_actionPerformed( null );
        }
    }

    public void componentHidden( ComponentEvent e )
    {}

    public void componentShown( ComponentEvent e )
    {}

    public void componentMoved( ComponentEvent e )
    {}

    public void componentResized( ComponentEvent e )
    {
        Dimension size = configData.getSize();
        size.width = getSize().width;
        size.height = getSize().height;
    }

}
