package sitool.components;

public class Cost
{
    private long iron;
    private long metal;
    private long kryptonite;
    private long spice;

    public Cost( long iron, long metal, long kryptonite, long spice )
    {
        super();
        this.iron = iron;
        this.metal = metal;
        this.kryptonite = kryptonite;
        this.spice = spice;
    }

    public Cost( Cost base, long multiplier )
    {
        super();
        this.iron = base.iron * multiplier;
        this.metal = base.metal * multiplier;
        this.kryptonite = base.kryptonite * multiplier;
        this.spice = base.spice * multiplier;
    }

    public long getIron()
    {
        return iron;
    }

    public long getMetal()
    {
        return metal;
    }

    public long getKryptonite()
    {
        return kryptonite;
    }

    public long getSpice()
    {
        return spice;
    }

    public boolean isOverflow()
    {
    	return ( iron < 0 || metal < 0 || kryptonite < 0 || spice < 0 );
    }

    public long getResource( Resource resource )
    {
        long rv = 0;
        if( resource != null )
        {
            switch( resource )
            {
                case iron:
                    rv = getIron();
                    break;
                case metal:
                    rv = getMetal();
                    break;
                case kryptonite:
                    rv = getKryptonite();
                    break;
                case spice:
                    rv = getSpice();
                    break;
            }
        }

        return rv;
    }

    public void setResource( Resource resource, long value )
    {
        if( resource != null )
        {
            switch( resource )
            {
                case iron:
                    iron = value;
                    break;
                case metal:
                    metal = value;
                    break;
                case kryptonite:
                    kryptonite = value;
                    break;
                case spice:
                    spice = value;
                    break;
            }
        }
    }

    public Cost add( Cost other )
    {
        iron += other.iron;
        metal += other.metal;
        kryptonite += other.kryptonite;
        spice += other.spice;
        return this;
    }

    public Cost multiply( long multiplier )
    {
        this.iron *= multiplier;
        this.metal *= multiplier;
        this.kryptonite *= multiplier;
        this.spice *= multiplier;
        return this;
    }

    public Cost multiply( double multiplier )
    {
        this.iron *= multiplier;
        this.metal *= multiplier;
        this.kryptonite *= multiplier;
        this.spice *= multiplier;
        return this;
    }

    public double points()
    {
        long all = iron + metal + kryptonite;
        return ((double)all) / 2000.0;
    }
}
