/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.util.HashMap;
import java.util.Map;

/**
 * @author graf
 *
 */
public class Fleet
{
    private Map<Ship, Long> ships = new HashMap<>();
    private Map<Defense, Long> defenses = new HashMap<>();

    public void resetShips() {
        ships = null;
    }

    public void resetDefenses() {
        defenses = null;
    }

    public boolean hasShips() {
        return ships != null;
    }

    public boolean hasDefenses() {
        return defenses != null;
    }

    /**
     * Returns the number of indicated ships
     * @param ship
     * @return
     */
    public long getCount( Ship ship )
    {
        int rv;
        Long count = (ships != null) ? ships.get( ship ) : null;
        rv = ( count != null ) ? count.intValue() : 0;
        return rv;
    }

    /**
     * Returns the number of indicated defenses
     * @param ship
     * @return
     */
    public long getCount( Defense defense )
    {
        int rv;
        Long count = (defenses != null) ? defenses.get( defense ) : null;
        rv = ( count != null ) ? count.intValue() : 0;
        return rv;
    }

    /**
     * Sets the number of ships
     * @param ship
     * @param count
     */
    public void setCount( Ship ship, Long count )
    {
        ships.put( ship, count );
    }

    /**
     * Sets the number of defenses
     * @param ship
     * @param count
     */
    public void setCount( Defense defense, Long count )
    {
        defenses.put( defense, count );
    }

    /**
     * Returns the overall velocity of the fleet, i.e. the speed of the slowest ship
     * @param version
     * @param tech
     * @param specialization
     * @param bonus an int between 0..100, the sum of all bonuses to the base speed, not from the first two
     * @return
     */
    public FleetSpeed getFleetVelocity( Version version, Technology technology, Specialization specialization, int bonus )
    {
        FleetSpeed lowestSpeed = new FleetSpeed();
        for( Ship ship : ships.keySet() )
        {
            // exclude stationary types
            if( ship.getVelocity() > 0 && getCount( ship ) > 0 )
            {
                if( version != Version.v20 )
                {
                    long velocity = (100 + bonus + specialization.getSpeedbonus(ship) + 10 * technology.getTech( ship.getType() ) ) * ship.getVelocity() / 100;
                    if( velocity < lowestSpeed.getVelocity() )
                    {
                        lowestSpeed.setVelocity(  velocity );
                        lowestSpeed.getShips()[FleetSpeed.VELOCITY] = ship;
                    }
                }
                else
                {
                    int acceleration = (10 + technology.getTech( ship.getType() ) ) * ship.getAcceleration() / 10;
                    long velocity = (20 + technology.getTech( ship.getType() ) ) * ship.getVelocity2() / 20;
                    if( velocity < lowestSpeed.getVelocity() )
                    {
                        lowestSpeed.setVelocity( velocity );
                        lowestSpeed.getShips()[FleetSpeed.VELOCITY] = ship;
                    }
                    if( acceleration < lowestSpeed.getAcceleration() )
                    {
                        lowestSpeed.setAcceleration( acceleration );
                        lowestSpeed.getShips()[FleetSpeed.ACCELERATION] = ship;
                    }
                    if( ship.getPreparationTime() > lowestSpeed.getPreptime() )
                    {
                        lowestSpeed.setPreptime( ship.getPreparationTime() );
                        lowestSpeed.getShips()[FleetSpeed.PREPERATIONTIME] = ship;
                    }
                }
            }
        }

        if( lowestSpeed.getVelocity() == Long.MAX_VALUE )
        {
            lowestSpeed.setVelocity( 0 );
            lowestSpeed.setAcceleration( 0 );
            lowestSpeed.setPreptime( 0 );
        }
        return lowestSpeed;
    }

    public void clearDefenses()
    {
        defenses.clear();
    }

    public void clearShips()
    {
        ships.clear();
    }

    public Map<Ship, Long> getShips() {
        return ships;
    }

    public Map<Defense, Long> getDefenses() {
        return defenses;
    }

    public void setShips(Map<Ship, Long> ships) {
        this.ships.clear();
        this.ships.putAll(ships);
    }

    public void setDefenses(Map<Defense, Long> defenses) {
        this.defenses.clear();
        this.defenses.putAll(defenses);
    }

    public void addShips(Map<Ship, Long> ships) {
        for(Map.Entry<Ship, Long> entry : ships.entrySet()) {
            long num = this.ships.containsKey(entry.getKey()) ? this.ships.get(entry.getKey()) : 0;
            num += entry.getValue();
            if(num != 0) {
                this.ships.put(entry.getKey(), num);
            }
        }
    }

    public void addDefenses(Map<Defense, Long> defenses) {
        for(Map.Entry<Defense, Long> entry : defenses.entrySet()) {
            long num = this.defenses.containsKey(entry.getKey()) ? this.defenses.get(entry.getKey()) : 0;
            num += entry.getValue();
            if(num != 0) {
                this.defenses.put(entry.getKey(), num);
            }
        }
    }
}
