/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.Stroke;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.border.Border;

/**
 * @author graf
 *
 */
public class OvalButton extends JButton
{
    private static final long serialVersionUID = -5415920702171297274L;
    
    private boolean armable = true;

    public OvalButton()
    {
        initialize();
    }

    public OvalButton( String text )
    {
        super( text );
        initialize();
    }

    @Override
    public void setOpaque( boolean isOpaque )
    {
        super.setOpaque( false );
    }

    public boolean isArmable()
    {
        return armable;
    }

    public void setArmable( boolean armable )
    {
        this.armable = armable;
    }

    @Override
    protected void paintComponent( Graphics g )
    {
        Color bg = getPaintColor();
        Dimension size = getSize();
        g.setColor( bg );
        g.fillOval( 3, 3, size.width - 7, size.height - 7 );

        super.paintComponent( g );
    }

    @Override
    protected void paintBorder( Graphics g )
    {
        Graphics2D g2 = ( Graphics2D )g;
        Color bg = getPaintColor();
        Color lighter = bg.brighter();
        Color darker = bg.darker().darker();
        Color upper, lower;
        if( armable && getModel().isArmed() )
        {
            upper = darker;
            lower = lighter;
        }
        else
        {
            upper = lighter;
            lower = darker;
        }
        Dimension size = getSize();
        float brush = (float)( Math.min( size.getWidth(), size.height ) / 10.0 );
        if( brush < 2.3f ) brush = 2.3f; 
        if( brush > 5.0f ) brush = 5.0f; 
        Paint borderPaint = new GradientPaint( size.width / 2, 0, upper, size.width / 2, size.height, lower, false );
        Stroke normal = new BasicStroke( brush, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL );
        g2.setPaint(borderPaint);
        g2.setStroke( normal );
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g.drawOval( 2, 2, size.width - 5, size.height - 5 );
    }

    @Override
    public void setBorder( Border border )
    {
        super.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );
    }
    
    private void initialize()
    {
        setBorder( null );
        setContentAreaFilled( false );
    }
    
    private Color getPaintColor()
    {
        Color color;
        if( getModel().isEnabled() )
        {
            color = getBackground();
        }
        else
        {
            color = Color.lightGray;
        }
        return color;
    }
}
