/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;

import sitool.ConfigData;


public enum Ship
{
    none( 0, Ship.Type.Rocketdrive, 0, 0, 0, 0, new Cost( 0, 0, 0, 0 ), 0, 0, 0, 0 ),
    smallTransport( 1, Ship.Type.Rocketdrive, 3000, 745, 2250, 1305000, new Cost( 0, 3000, 3000, 0 ), 10, 2, 6000, 10 ),
    largeTransport( 2, Ship.Type.Rocketdrive, 5000, 550, 2250, 1698000, new Cost( 0, 8000, 6000, 0 ), 25, 5, 30000, 15 ),
    transmitter( 3, Ship.Type.Diffusiondrive, 7000, 470, 3000, 2005500, new Cost( 0, 25000, 25000, 0 ), 70, 130, 250000, 25 ),
    smallFighter( 4, Ship.Type.Rocketdrive, 9000, 400, 3000, 2400000, new Cost( 0, 4000, 2000, 100 ), 10, 50, 60, 15 ),
    manOWar( 5, Ship.Type.Rocketdrive, 6500, 455, 2250, 1957500, new Cost( 0, 8000, 4000, 300 ), 30, 90, 100, 28  ),
    starCruiser( 6, Ship.Type.Diffusiondrive, 4800, 550, 2250, 1657500, new Cost( 0, 25000, 10000, 1500 ), 100, 250, 200, 40 ),
    phoenix( 7, Ship.Type.Diffusiondrive, 8000, 410, 2250, 2250000, new Cost( 0, 30000, 20000, 5000 ), 180, 320, 500, 55 ),
    colonyShip( 8, Ship.Type.Rocketdrive, 1500, 1035, 9000, 930000, new Cost( 10000, 20000, 15000, 8000 ), 80, 10, 5000, 32 ),
    solarCell( 9, Ship.Type.Rocketdrive, 0, 0, 0, 0, new Cost( 0, 2000, 200, 400 ), 5, 0, 0, 0 ),
    probe( 10, Ship.Type.Rocketdrive, 15000000, 10, 3000000, 4516065000L, new Cost( 0, 0, 3000, 200 ), 6, 0, 6, 3 ),
    recycler( 11, Ship.Type.Diffusiondrive, 2000, 380, 675, 1125000, new Cost( 2000, 10000, 5000, 0 ), 30, 8, 25000, 20 ),
    battleship( 13, Ship.Type.Warpdrive, 6800, 1000, 2100, 2827500,  new Cost( 0, 35000, 25000, 0 ), 300, 300, 1000, 90 ),
    stealthBomber( 14, Ship.Type.Warpdrive, 5200, 530, 1950, 1725000, new Cost( 0, 40000, 60000, 10000 ), 100, 300, 1400, 110 ),
    destroyer( 15, Ship.Type.Warpdrive, 4000, 620, 1950, 1515000, new Cost( 0, 60000, 50000, 2000 ), 400, 700, 3000, 130 ),
    imperialStarbase( 16, Ship.Type.Warpdrive, 300, 2460, 4500, 412500 , new Cost( 0, 3000000, 2000000, 500000 ), 15000, 35000, 3500000, 250 ),
    jammer( 17, Ship.Type.Rocketdrive, -1, 10, 3000000, 4516065000L, new Cost( 0, 0, 6000, 400 ), 6, 0, 6, 3 ),
    empbomb( 18, Ship.Type.Rocketdrive, -1, 10, 1200000, 4516065000L, new Cost( 0, 2000, 200, 200 ), 6, 0, 6, 3 ),
    scanship( 19, Ship.Type.Rocketdrive, -1, 400, 3000, 2400000, new Cost( 0, 4000, 2000, 100 ), 10, 0, 60, 15 ),
    ;

    private Type type;
    private long velocity;
    private int preparationTime;
    private int acceleration;
    private long velocity2;
    private Cost cost;
    private int firePower;
    private int shields;
    private int load;
    private int consumption;
    private final Integer id;
    private static final Map<Integer, Ship> reverseMap;

    static {
        reverseMap = new HashMap<>();
        for(Ship Ship : values()) {
            reverseMap.put(Ship.id, Ship);
        }
    }

    public static Ship fromId(String id) {
        int i = Integer.parseInt(id.replace("s", ""));
        return reverseMap.get(Integer.valueOf(i));
    }


    private Ship( int id, Type type, long velocity, int preparationTime, int acceleration, long velocity2, Cost cost, int shields, int firePower, int load, int consumption )
    {
        this.type = type;
        this.velocity = velocity;
        this.preparationTime = preparationTime;
        this.acceleration = acceleration;
        this.velocity2 = velocity2;
        this.cost = cost;
        this.firePower = firePower;
        this.shields = shields;
        this.load = load;
        this.consumption = consumption;
        this.id = Integer.valueOf(id);
    }

    public String getId() {
        return id.toString();
    }

    public Type getType()
    {
        return type;
    }

    public long getVelocity()
    {
        return velocity;
    }

    public int getPreparationTime()
    {
        return preparationTime;
    }

    public int getAcceleration()
    {
        return acceleration;
    }

    public long getVelocity2()
    {
        return velocity2;
    }

    /**
     * @return the cost
     */
    public Cost getCost()
    {
        return cost;
    }

    /**
     * @return the firePower
     */
    public int getFirePower()
    {
        return firePower;
    }

    /**
     * @return the shields
     */
    public int getShields()
    {
        return shields;
    }

    /**
     * @return the load
     */
    public int getLoad()
    {
        return load;
    }

    /**
     * @return the consumption
     */
    public int getConsumption()
    {
        return consumption;
    }

    public boolean isSupported( Version version )
    {
        boolean rv = true;
        if( version != Version.v20 )
        {
            switch( this )
            {
                case jammer:
                case empbomb:
                case scanship:
                    rv = false;
                    break;
            }
        }
        else
        {
            switch( this )
            {
                case solarCell:
                    rv = false;
                    break;
            }
        }
        return rv;
    }

    public static Ship[] ships(Version version)
    {
        EnumSet<Ship> ships = EnumSet.range( Ship.smallTransport, Ship.scanship );
        for( Ship ship : ships )
        {
            if( !ship.isSupported( version ) )
            {
                ships.remove( ship );
            }
        }
        return ships.toArray( new Ship[0] );
    }

    public static Ship[] movableShips(Version version)
    {
        List<Ship> ships = new ArrayList<Ship>( Arrays.asList( ships( version ) ) );
        ships.remove( Ship.solarCell );
        return ships.toArray( new Ship[0] );
    }


    static public enum Type
    {
        Rocketdrive,
        Diffusiondrive,
        Warpdrive;
    }

    public Icon getIcon()
    {
        Icon rv = ConfigData.getIcon( "/ship_" + getId() + ".gif" );
        return rv;
    }
}
