/****************************************
 *  COPYRIGHT (C) 2022
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;

import sitool.ConfigData;

/**
 * @author graf
 *
 */
public class SpecializationBox extends JDialog {

    private static final long serialVersionUID = -8399215445703695460L;

    private JLabel l_miner = new JLabel();
    private JLabel l_fleeter = new JLabel();
    private JLabel l_researcher = new JLabel();

    private JSpinner s_miner = new JSpinner(new SpinnerNumberModel(0, 0, 4, 1));
    private JSpinner s_fleeter = new JSpinner(new SpinnerNumberModel(0, 0, 4, 1));
    private JSpinner s_researcher = new JSpinner(new SpinnerNumberModel(0, 0, 4, 1));

    private JButton okButton = new JButton("wwwwww");
    private JButton cancelButton = new JButton("wwwwwwwww");

    private TableCellEditor editor = null;

    private final Specialization specialization = new Specialization();
    private final ConfigData configData;

    public SpecializationBox( ConfigData configData, Dialog owner, boolean modal ) {
        super(owner, modal);
        this.configData = configData;
        setIconImage(new ImageIcon(getClass().getResource( "/symbol.png" )).getImage());
        setDefaultCloseOperation(HIDE_ON_CLOSE);
        getContentPane().setLayout(new BorderLayout(5, 5));
        ((JPanel)getContentPane()).setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        JPanel centerPane = new JPanel(new GridLayout(3, 2, 5, 5));
        centerPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        centerPane.add(l_miner);
        centerPane.add(s_miner);
        centerPane.add(l_fleeter);
        centerPane.add(s_fleeter);
        centerPane.add(l_researcher);
        centerPane.add(s_researcher);
        getContentPane().add(centerPane, BorderLayout.CENTER);
        JPanel buttonPane = new JPanel(new GridLayout(1, 2, 5, 5));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 15, 5, 15));
        okButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed( ActionEvent e ) {
                ok();
            }
        });
        cancelButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed( ActionEvent e ) {
                cancel();
            }
        });
        buttonPane.add(okButton);
        buttonPane.add(cancelButton);
        getContentPane().add( buttonPane, BorderLayout.SOUTH );
        pack();
    }

    public void show(TableCellEditor editor, Specialization specialization, Point point) {
        this.editor = editor;
        this.specialization.setMiner(specialization.getMiner());
        this.specialization.setFleeter(specialization.getFleeter());
        this.specialization.setResearcher(specialization.getResearcher());
        s_miner.setValue(specialization.getMiner());
        s_fleeter.setValue(specialization.getFleeter());
        s_researcher.setValue(specialization.getResearcher());
        updateLabel();
        setLocation(point);
        setVisible(true);
    }

    public void cancel() {
        setVisible(false);
        if(editor != null) {
            editor.cancelCellEditing();
        }
        editor = null;
    }

    public Specialization getSpecialization() {
        return specialization;
    }

    private void ok() {
        specialization.setMiner(((Number)s_miner.getValue()).intValue());
        specialization.setFleeter(((Number)s_fleeter.getValue()).intValue());
        specialization.setResearcher(((Number)s_researcher.getValue()).intValue());
        setVisible(false);
        if(editor != null) {
            editor.stopCellEditing();
        }
        editor = null;
    }

    private void updateLabel() {
        setTitle(configData.getI18nText("Account.specialization"));
        l_miner.setText(configData.getI18nText("Account.miner"));
        l_fleeter.setText(configData.getI18nText("Account.fleeter"));
        l_researcher.setText(configData.getI18nText("Account.researcher"));
        okButton.setText(configData.getI18nText("OK"));
        cancelButton.setText(configData.getI18nText("Cancel"));
    }
}
