package sitool.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.FocusListener;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import sitool.ConfigData;

public class TimeField extends JComponent
{
	private static final long serialVersionUID = 7486663925998614807L;
    private JTextField dayField = new JTextField( 3 );
    private JTextField hourField = new JTextField( 3 );
    private JTextField minuteField = new JTextField( 3 );
    private JLabel daySymbol;
    private ConfigData configData;

	public TimeField( ConfigData configData )
	{
		super();
		super.setLayout( new GridBagLayout() );
		this.configData = configData;

        dayField.setHorizontalAlignment( SwingConstants.RIGHT );
        hourField.setHorizontalAlignment( SwingConstants.RIGHT );
        minuteField.setHorizontalAlignment( SwingConstants.RIGHT );

        // initialize constraints
		GridBagConstraints fieldConstraints = new GridBagConstraints();
        fieldConstraints.gridheight = 1;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.fill = GridBagConstraints.BOTH;
        fieldConstraints.anchor = GridBagConstraints.CENTER;
        fieldConstraints.weighty = 1.0;

        fieldConstraints.weightx = 1.0;
        add( dayField, fieldConstraints );
        fieldConstraints.weightx = 0.0;
        daySymbol = new JLabel( configData.getI18nText( "day.symbol" ) );
        add( daySymbol, fieldConstraints );
        fieldConstraints.weightx = 1.0;
        add( hourField, fieldConstraints );
        fieldConstraints.weightx = 0.0;
        add( new JLabel( ":" ), fieldConstraints );
        fieldConstraints.weightx = 1.0;
        add( minuteField, fieldConstraints );
        fieldConstraints.weightx = 0.0;
        
        FocusListener l = new TextFieldFocusListener();
        dayField.addFocusListener( l );
        hourField.addFocusListener( l );
        minuteField.addFocusListener( l );
	}

    public void addFocusListener( FocusListener l )
    {
        dayField.addFocusListener( l );
        hourField.addFocusListener( l );
        minuteField.addFocusListener( l );
    }
    
    public void removeFocusListener( FocusListener l )
    {
        dayField.removeFocusListener( l );
        hourField.removeFocusListener( l );
        minuteField.removeFocusListener( l );
    }
    
	public void update()
	{
	    daySymbol.setText( configData.getI18nText( "day.symbol" ) );
	}
	
    public String getDay()
    {
        return dayField.getText();
    }

    public void setDay( String day )
    {
        dayField.setText( day );
    }

    public String getHour()
    {
        return hourField.getText();
    }

    public void setHour( String hour )
    {
        hourField.setText( hour );
    }

	public String getMinute()
	{
        return minuteField.getText();
	}

	public void setMinute( String minute )
	{
        minuteField.setText( minute );
	}

}
