/****************************************
 *  COPYRIGHT (C) 2006
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.Timer;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.SoftBevelBorder;

import sitool.ConfigData;
import sitool.savecalc.SaveTarget;
import sitool.table.SaveTargetTableModel;

/**
 * @author graf
 *
 */
public class TimePanel extends JPanel
{
    private static final long serialVersionUID = -6670962184790495392L;

    private static final SimpleDateFormat time = new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" );

    private ConfigData configData;

    private JLabel labelCurrentTime = new JLabel();
    private JLabel labelOnewayTime = new JLabel();
    private JLabel labelReturnTime = new JLabel();

    private JLabel currentTime = new JLabel();
    private JLabel onewayTime = new JLabel();
    private JLabel returnTime = new JLabel();

    private JTable selectedTargets;
    
    public TimePanel( ConfigData configData, JTable table )
    {
        super( new GridLayout( 6, 1, 5, 5 ) );
        this.configData = configData;
        this.selectedTargets = table;
        
        Border border = new CompoundBorder(
                        new CompoundBorder( 
                                        new EmptyBorder( 5, 4, 5, 4 ),
                                        new SoftBevelBorder( SoftBevelBorder.RAISED )
                        ),
                        new EmptyBorder( 10, 10, 10, 10 )
        );
        this.setBorder( border );
        //timePanel.setBackground(Color.red);
        Font font = this.getFont().deriveFont( Font.BOLD );
    
        labelCurrentTime.setFont( font );
        this.add( labelCurrentTime );
        currentTime.setFont(font);
        currentTime.setOpaque( true );
        currentTime.setBackground( Color.white );
        currentTime.setHorizontalAlignment( SwingConstants.CENTER );
        this.add( currentTime );
        labelOnewayTime.setFont( font );
        this.add( labelOnewayTime );
        onewayTime.setFont(font);
        onewayTime.setOpaque( true );
        onewayTime.setBackground( Color.white );
        onewayTime.setHorizontalAlignment( SwingConstants.CENTER );
        this.add( onewayTime );
        labelReturnTime.setFont( font );
        this.add( labelReturnTime );
        returnTime.setFont( font );
        returnTime.setOpaque( true );
        returnTime.setBackground( Color.white );
        returnTime.setHorizontalAlignment( SwingConstants.CENTER );
        this.add( returnTime );
        this.setMinimumSize( this.getPreferredSize() );
        this.setMaximumSize( this.getPreferredSize() );
        
        int delay = 1000; //milliseconds
        ActionListener task = new ActionListener() 
        {
            public void actionPerformed(ActionEvent e)
            {
                actionTimer();
            }
        };
        // after lengthly perusal of available documentation
        // javax.swing.Timer was chosen over java.util.Timer
        new Timer( delay, task ).start();
    }

    protected void actionTimer()
    {
        long now = System.currentTimeMillis();
        currentTime.setText( time.format( new Date( now ) ) );
        if( selectedTargets.getSelectedRowCount() == 1 )
        {
            if( selectedTargets.getModel() instanceof SaveTargetTableModel )
            {
                SaveTargetTableModel model = ( SaveTargetTableModel )selectedTargets.getModel();
                SaveTarget saveTarget = model.getRow( selectedTargets.getSelectedRow() );
                long duration = saveTarget.getDuration();
                onewayTime.setText( time.format( new Date( now + 1000 * duration ) ) );
                returnTime.setText( time.format( new Date( now + 2000 * duration ) ) );
            }
        }
        else
        {
            onewayTime.setText( "" );
            returnTime.setText( "" );
        }
    }

    public void updateLabel()
    {
        labelCurrentTime.setText( configData.getI18nText( "Savecalc.currentTime" ) );
        labelOnewayTime.setText( configData.getI18nText( "Savecalc.onewayTime" ) );
        labelReturnTime.setText( configData.getI18nText( "Savecalc.returnTime" ) );
    }
    
    public static DateFormat getDateFormat()
    {
    	return time;
    }
}
