/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.text.DecimalFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;

/**
 * @author graf
 *
 */
public class TotalPointDisplay extends JPanel implements TableModelListener
{
    private static final long serialVersionUID = -3601777735176367071L;

    private MainFrame mainFrame;
    private ConfigData configData;

    private long totalPoints;

    private JLabel total = new JLabel();
    private JLabel buildings = new JLabel();
    private JLabel research = new JLabel();
    private JLabel fleet = new JLabel();
    private JLabel tactical = new JLabel();

    private JLabel labelTotal = new JLabel();
    private JLabel labelBuildings = new JLabel();
    private JLabel labelResearch = new JLabel();
    private JLabel labelFleet = new JLabel();
    private JLabel labelTactical = new JLabel();
    private JLabel connectionState = new JLabel();

    private static final char groupingSeparator = new DecimalFormat().getDecimalFormatSymbols().getGroupingSeparator();

    public TotalPointDisplay( MainFrame mainFrame)
    {
        super();
        this.mainFrame = mainFrame;
        configData = mainFrame.getConfigData();

        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder() );
        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    public long getTotalPoints()
    {
        return totalPoints;
    }

    public void tableChanged( TableModelEvent e )
    {
        long fleet = 0;
        totalPoints = 0;

        for( TotalPointSource.Type type : TotalPointSource.Type.values() )
        {
            TotalPointSource source = mainFrame.getTotalPointSource( type );
            long value  = ( source != null ) ? source.getTotalPoints() : 0;
            switch( type )
            {
                case Buildings:
                    totalPoints += value;
                    this.buildings.setText( format( value ) );
                    break;
                case Research:
                    totalPoints += value;
                    this.research.setText( format( value ) );
                    break;
                case Tactical:
                    this.tactical.setText( format( value ) );
                    break;
                case Ships:
                case Defense:
                case AstroShips:
                case AstroDefense:
                    totalPoints += value;
                    fleet += value;
                    break;

            }
        }
        this.fleet.setText( format( fleet ) );
        this.total.setText( format( totalPoints ) );
        this.repaint();
    }

    public void update()
    {
        Account current = mainFrame.getConfigData().getActiveAccount();
        if( current != null )
        {
            setVisible( true );
            tableChanged( null );
            updateLabel();
            boolean tacticalVisible = false; // current.hasTotalShips();
            labelTactical.setVisible(tacticalVisible);
            tactical.setVisible(tacticalVisible);
        }
        else
        {
            setVisible( false );
        }
    }

    private void updateLabel()
    {
        labelBuildings.setText( configData.getI18nText( "Points.buildings" ) );
        labelResearch.setText( configData.getI18nText( "Points.research" ) );
        labelFleet.setText( configData.getI18nText( "Points.fleet" ) );
        labelTactical.setText( configData.getI18nText( "Points.tactical" ) );
        labelTotal.setText( configData.getI18nText( "Points.total" ) );
        String path = new StringBuilder("/connection-")
                    .append(mainFrame.isDataServerRunning() ? "on" : "off")
                    .append(".png")
                    .toString();
        connectionState.setIcon(ConfigData.getIcon(path, new Dimension(16, 16)));
    }

    public static String format( long value )
    {
        StringBuilder sb = new StringBuilder();
        boolean negative = (value < 0 );
        sb.append( value );
        for( int i = sb.length() - 3; i > 0; i-= 3)
        {
        	if( !negative || i > 1 )
        	{
        		sb.insert( i, groupingSeparator );
        	}
        }
        return sb.toString();
    }

    private JPanel createCenterPanel()
    {
        Color fg = Color.blue.darker();
        JPanel center = new JPanel();
        connectionState.setOpaque(true);
        connectionState.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
        JLabel label = new JLabel( "WWW.WWW.WWW" );
        Dimension size = label.getPreferredSize();
        buildings.setPreferredSize( size );
        buildings.setHorizontalAlignment( SwingConstants.RIGHT );
        buildings.setBorder( BorderFactory.createEtchedBorder() );
        research.setPreferredSize( size );
        research.setHorizontalAlignment( SwingConstants.RIGHT );
        research.setBorder( BorderFactory.createEtchedBorder() );
        fleet.setPreferredSize( size );
        fleet.setHorizontalAlignment( SwingConstants.RIGHT );
        fleet.setBorder( BorderFactory.createEtchedBorder() );
        total.setPreferredSize( size );
        total.setHorizontalAlignment( SwingConstants.RIGHT );
        total.setBorder( BorderFactory.createEtchedBorder() );
        total.setForeground( fg );

        center.setLayout( new GridLayout( 2, 6, 0, 0 ) );
        labelBuildings.setBorder( BorderFactory.createEtchedBorder() );
        center.add( labelBuildings );
        center.add( buildings );
        labelResearch.setBorder( BorderFactory.createEtchedBorder() );
        center.add( labelResearch );
        center.add( research );
        labelFleet.setBorder( BorderFactory.createEtchedBorder() );
        center.add( labelFleet );
        center.add( fleet );
        JPanel connectionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
        connectionPanel.add( connectionState );
        center.add( connectionPanel );
        label = new JLabel( "" );
        center.add( label );
        labelTotal.setForeground( fg );
        labelTotal.setBorder( BorderFactory.createEtchedBorder() );
        center.add( labelTotal );
        center.add( total );
        labelTactical.setBorder( BorderFactory.createEtchedBorder() );
        center.add( labelTactical );
        tactical.setBorder( BorderFactory.createEtchedBorder() );
        tactical.setHorizontalAlignment( SwingConstants.RIGHT );
        tactical.setPreferredSize(size);
        center.add( tactical );

        return center;
    }
}
