/****************************************
 *  COPYRIGHT (C) 2012-2024
 *  Holger Graf
 ****************************************/
package sitool.components;

/**
 * @author graf
 *
 */
public enum Version
{
    nexus(44, 8, 400, 360, 14, false), retro(31, 8, 400, 360, 14, false), sirius(46, 6, 400, 200, 14, false),
    genesis(40, 10, 200, 0, 14, true), v20(36, 6, 400, 0, 0, false), speed(35, 8, 400, 0, 0, false),
    solaris(45, 2, 400, 0, 14, false), infinity(47, 5, 400, 360, 14, false);

    private final int upperGalaxyBound;
    private final int upperSystemBound;
    private final int id;
    private final long cap;
    private final int outpostGalaxy;
    private final boolean hasSpectral;

    private Version(final int id, final int upperGalaxyBound, final int upperSystemBound,
                    final int cap, final int outpostGalaxy, final boolean hasSpectral) {
        this.id = id;
        this.upperGalaxyBound = upperGalaxyBound;
        this.upperSystemBound = upperSystemBound;
        this.cap = cap * 24l * 60l * 60l * 1000l;
        this.outpostGalaxy = outpostGalaxy;
        this.hasSpectral = hasSpectral;
    }

    public int getId() {
        return id;
    }

    public int getUpperGalaxyBound() {
        return upperGalaxyBound;
    }

    public int getUpperSystemBound() {
        return upperSystemBound;
    }

    public long getResearchCap() {
        return cap;
    }

    public int getOutpostGalaxy() {
        return outpostGalaxy;
    }

    public boolean isSpectral() {
        return hasSpectral;
    }
}
