/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.fleet;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class FleetPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 2758582246555940481L;
    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JTabbedPane pane;

    private Map<String, Integer> titles = new HashMap<String, Integer>();

    public FleetPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( null );

        add( createCenterPanel(), BorderLayout.CENTER );
    }

    public void update()
    {
        // disable tactical fleet strength pane for SI2
        boolean enabled = configData.getVersion() != Version.v20;
        int tactical = titles.get("fleet.tactical");
        pane.setEnabledAt( tactical, enabled );
        if( !enabled && tactical == pane.getSelectedIndex() ) {
            pane.setSelectedIndex( 0 );
        }

        // propagate update
        Component c = pane.getSelectedComponent();
        if( c instanceof ModulePane )
        {
            ((ModulePane)c).update();
            //System.out.println( "update pane " + c.getClass().getSimpleName() );
        }
        for( Map.Entry<String, Integer> entry : titles.entrySet() )
        {
            int index = entry.getValue().intValue();
            pane.setTitleAt( index, configData.getI18nText( entry.getKey() ) );
        }
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );

        String key;
        int index = 0;

        pane = new JTabbedPane( JTabbedPane.TOP );
        SummaryShipPane ships = new SummaryShipPane( mainFrame );
        key = "fleet.ships";
        pane.add( ships, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        SummaryDefensePane defense = new SummaryDefensePane( mainFrame );
        key = "fleet.defense";
        pane.add( defense, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        TacticalStrengthPane tactical = new TacticalStrengthPane( mainFrame );
        key = "fleet.tactical";
        pane.add( tactical, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        FleetBuilderPane builder = new FleetBuilderPane( mainFrame );
        key = "fleet.fleetbuilder";
        pane.add( builder, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        pane.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                update();
            }
        });
        pane.setSelectedIndex( 0 );
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

}
