/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.fleet;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import com.fasterxml.jackson.jr.ob.JSON;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.Ship;
import sitool.components.Version;
import sitool.datatransfer.DataImport;
import sitool.json.IntergalacticOverviewBean;
import sitool.json.JSONTags;

/**
 *
 * @author graf
 *
 */
public class ShipBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -5934119091136695023L;
    private static final String split = "\\s+";

    private JTextArea textArea = new JTextArea();

    private MainFrame mainFrame = null;
    private ConfigData configData;
    private Version version;
    private Planet planet = null;
    private Pattern numberPattern = Pattern.compile( "\\(\\D+(\\d+)\\D+(\\d+)\\D*\\)" );
    private Pattern numberPattern2 = Pattern.compile( "\\[(\\d+)\\]\\s+(\\[.+:\\s+(\\d+)\\])?" );
    private Pattern linePattern2 = Pattern.compile( "\\s+\\((\\d+)\\)" );


	public ShipBox( MainFrame parent, Planet target )
    {
	    super( parent );
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        version = configData.getVersion();
        planet = target;

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "ShipBox.title", target.toString() ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 640, 520 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();

        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {

            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    textArea.setText( input );
                }
            }

        };

        textArea.addMouseListener( l );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JScrollPane scrollPane = new JScrollPane( textArea,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        center.add( scrollPane, BorderLayout.CENTER );

        return center;
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        String input = textArea.getText();
        if( input.indexOf(JSONTags.reportType_PlanetOverview) > -1 ) {
            scanIntergalacticPlanetOverviewJSON(input);
        }
        else {
            Fleet fleet = planet.getFleet();
            boolean weapons = false;
            Pattern pattern = Pattern.compile( configData.getI18nText( "ShipBox.view.marker" ) );
            weapons = pattern.matcher( input ).find();
            BufferedReader br = new BufferedReader( new StringReader( input ) );
            String line;

            fleet.clearShips();
            try
            {
                while( (line = br.readLine() ) != null )
                {
                    if( version != Version.v20 )
                    {
                        if( weapons )
                        {
                            scanLineWeapons( fleet, line );
                        }
                        else
                        {
                            if( scanLineFleet( fleet, line ) )
                            {
                                break;
                            }
                        }
                    }
                    else
                    {
                        scanLine2( fleet, line, weapons );
                    }
                }
                br.close();
            }
            catch( IOException e1 )
            {}
        }
        mainFrame.actionUpdatePane();
        dispose();
    }

    private void scanIntergalacticPlanetOverviewJSON( String input ) {
        try
        {
            IntergalacticOverviewBean overview = JSON.std.beanFrom(IntergalacticOverviewBean.class, input);
            List<Planet> planets = DataImport.createPlanetsFromJSON(version, overview, configData.isOutpost(), false, true);
            int index = planets.indexOf(planet);
            if( index >= 0 ) {
                Planet newPlanet = planets.get(index);
                planet.update(newPlanet);
            }
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }
    }

    private boolean scanLineFleet( Fleet fleet, String line )
    {
        boolean rv = ( line.indexOf( configData.getI18nText( "ShipBox.end.marker" ) ) >= 0 );
        if( !rv )
        {
            // scan for ships
            for( Ship ship : Ship.values() )
            {
                if( ship != Ship.none && line.indexOf( configData.getI18nText( ship.name() ) ) >= 0 )
                {
                    String[] parts = line.split( split, 3 );
                    if( parts.length > 1 )
                    {
                        long count = 0;
                        try
                        {
                            count = Long.parseLong( parts[0].trim() );
                        }
                        catch( NumberFormatException ex )
                        {}
                        if( count > 0 )
                        {
                            fleet.setCount( ship, count );
                            break;
                        }
                    }
                }
            }
        }
        return rv;
    }

    private void scanLineWeapons( Fleet fleet, String line )
    {
        for( Ship ship : Ship.values() )
        {
            if( ship != Ship.none && line.indexOf( configData.getI18nText( ship.name() ) ) >= 0 )
            {
                Matcher m = numberPattern.matcher( line );
                if( m.find() )
                {
                    long count = Long.parseLong( m.group( 1 ) );
                    fleet.setCount( ship, count );
                    break;
                }
            }
        }
    }

    private void scanLine2( Fleet fleet, String line, boolean weapons )
    {
        for( Ship ship : Ship.values() )
        {
            if( ship != Ship.none && line.indexOf( configData.getI18nText( ship.name() ) ) >= 0 )
            {
                Matcher m = (weapons) ? numberPattern2.matcher( line ) : linePattern2.matcher( line );
                if( m.find() )
                {
                    long count = Long.parseLong( m.group( 1 ) );
//                    if( weapons && m.group( 3 ) != null )
//                    {
//                        count -= Long.parseLong( m.group( 3 ) );
//                    }
                    fleet.setCount( ship, count );
                    break;
                }
            }
        }
    }
}
