/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.planets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.CoordinatorTableModel;
import sitool.table.NumberTableCellRenderer;
import sitool.table.PlanetTableModel;
import sitool.table.TemperatureTableCellRenderer;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class CoordinatorPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private MainFrame mainFrame = null;
    private ConfigData configData;

    private CoordinatorTableModel planets;
    private JTable planetTable;

    private JLabel labelGalaxyTicks;
    private JSpinner galaxyTicks;
    private JLabel labelTicks;
    private JSpinner ticks;
    private JLabel syncLabel;
    private JButton syncButton;
//    private JTextField importField;
//    private JButton importButton;
    private JButton filterButton;
    private JButton filterMyPlanets;
    private JTextField filterField;

    private int currentGalaxyTicks;
    private int currentTicks;

	public CoordinatorPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );

        JPanel northPanel = createNorthPanel();
        add(  northPanel, BorderLayout.NORTH );

        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    private JPanel createNorthPanel()
    {
        JPanel north = new JPanel( new BorderLayout() );
        JPanel tickPanel = new JPanel( new FlowLayout( FlowLayout.LEFT, 15, 5 ) );
        JPanel syncPanel = new JPanel( new FlowLayout( FlowLayout.RIGHT, 15, 5 ) );
        labelGalaxyTicks = new JLabel();
        galaxyTicks = new JSpinner( new SpinnerNumberModel( 0, 0, 99999999, 1 ) );
        galaxyTicks.addChangeListener( new ChangeListener()
        {
            @Override
            public void stateChanged( ChangeEvent e )
            {
                currentGalaxyTicks = ((Integer)galaxyTicks.getValue()).intValue();
                planets.setTicks( currentGalaxyTicks, currentTicks );
                updateSync( configData.getActiveAccount() );
            }
        } );
        labelTicks = new JLabel();
        ticks = new JSpinner( new SpinnerNumberModel( 0, 0, 99999999, 1 ) );
        ticks.addChangeListener( new ChangeListener()
        {
            @Override
            public void stateChanged( ChangeEvent e )
            {
                currentTicks = ((Integer)ticks.getValue()).intValue();
                planets.setTicks( currentGalaxyTicks, currentTicks );
                updateSync( configData.getActiveAccount() );
            }
        } );
        syncLabel = new JLabel() {
            private static final long serialVersionUID = -4912462827902669661L;

            @Override
            public Dimension getPreferredSize()
            {
                Dimension size = super.getPreferredSize();
                size.width = 200;
                size.height += 9;
                return size;
            }
        };
        syncLabel.setOpaque( true );
        syncLabel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
        syncLabel.setHorizontalAlignment( SwingConstants.CENTER );
        syncLabel.addMouseListener( new MouseAdapter()
        {
            @Override
            public void mouseClicked( MouseEvent e )
            {
                if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
                {
                    setSync( currentGalaxyTicks, currentTicks );
                }
            }

            @Override
            public void mousePressed( MouseEvent e )
            {
                syncLabel.setBorder( BorderFactory.createBevelBorder( BevelBorder.LOWERED ) );
            }

            @Override
            public void mouseReleased( MouseEvent e )
            {
                syncLabel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
            }
        } );
        syncButton = new JButton();
        syncButton.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                actionSync();
            }
        } );

        tickPanel.add( labelGalaxyTicks );
        tickPanel.add( galaxyTicks );
        tickPanel.add( labelTicks );
        tickPanel.add( ticks );
        syncPanel.add( syncLabel );
        syncPanel.add( syncButton );

        north.add( tickPanel, BorderLayout.WEST );
        north.add( syncPanel, BorderLayout.EAST );
        return north;
    }

    protected void setSync( int currentGalaxyTicks, int currentTicks )
    {
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            current.setTicks( currentTicks );
            current.setGalaxyTicks( currentGalaxyTicks );
            current.setSync( true );
            configData.adjustCoordinates();
            update();
            mainFrame.updateIndicator();
        }
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel controlPanel = new JPanel( new BorderLayout() );
        JPanel lowerControlPanel = new JPanel( new BorderLayout() );
        lowerControlPanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel filterPanel = new JPanel( new FlowLayout( FlowLayout.LEFT, 0, 1 ) );
        filterButton = new JButton();
        filterButton.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                actionFilter();
            }
        } );
        filterMyPlanets = new JButton();
        filterMyPlanets.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                filterField.setText( createFilterString() );
                actionFilter();
            }
        } );
        filterField = new JTextField( 25 );
        filterPanel.add( filterField );
        filterPanel.add( filterButton );
        filterPanel.add( filterMyPlanets );
        lowerControlPanel.add( filterPanel, BorderLayout.WEST );
        controlPanel.add( lowerControlPanel, BorderLayout.SOUTH );

        center.add( controlPanel, BorderLayout.NORTH );


        planets = new CoordinatorTableModel( configData );
        final TableCellRenderer col3Renderer = new TemperatureTableCellRenderer();
        planetTable = new TooltipTable( planets ) {
            private static final long serialVersionUID = 2885181790454689168L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case PlanetTableModel.TEMPERATURE:
                        rv = col3Renderer;
                        break;
                    default:
                        rv = super.getCellRenderer( row, column );
                        break;
                }
                return rv;
            }
        };
        planetTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        TableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        planetTable.setDefaultRenderer( String.class, defaultCellRenderer );
        planetTable.setDefaultRenderer( Number.class, new NumberTableCellRenderer() );

        JScrollPane scrollPane = new JScrollPane( planetTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        center.add( scrollPane, BorderLayout.CENTER );

        return center;
    }

    protected void actionFilter()
    {
        planets.filter( filterField.getText() );
    }

    protected String createFilterString()
    {
        StringBuilder sb = new StringBuilder();
        List<Planet> planets = configData.getMyPlanets();
        for( Planet planet : planets )
        {
            sb.append( "(\\" );
            sb.append( planet.getPosition().toString() );
            sb.append( ")|" );
        }
        if( sb.length() > 0 )
        {
            sb.deleteCharAt( sb.length() - 1 );
        }
        String filter = sb.toString();
        return filter.replace( "]", "\\]" );
    }

//    Pattern xmlPattern = Pattern.compile( Parser.Attribute_Orbit + "=\"(\\d+)\"\\s+" + Parser.Attribute_Position + "=\"\\[?(\\d+:\\d+:\\d+)\\]?\"\\s+" + Parser.Attribute_Start + "=\"(\\d+)\"");
//    protected void actionImportPlanet()
//    {
//        String line = importField.getText();
//        Matcher m = xmlPattern.matcher( line );
//        if( m.find() )
//        {
//            importPlanet( m );
//        }
//        else
//        {
//            importPlanet( line );
//        }
//        planets.update();
//        importField.setText( "" );
//    }

//    private void importPlanet( Matcher m )
//    {
//        m.reset();
//        Map<Position, Coordinates> coordinates = configData.getCoordinates( configData.getActiveAccount() );
//        while( m.find() )
//        {
//            int orbit = ConfigData.parseIntegerAttribute( m.group( 1 ) );
//            Position position = new Position( m.group( 2 ) );
//            int start = ConfigData.parseIntegerAttribute( m.group( 3 ) );
//            Coordinates c = new Coordinates( position, start, orbit );
//            coordinates.put( position, c );
//        }
//    }
//
//    private void importPlanet(String line)
//    {
//        Pattern pattern = Pattern.compile( "(\\[?\\d+:\\d+:\\d+\\]?\\s+(\\S\\s+)?\\d+:\\d+\\|\\d+:\\d+\\|\\d+:\\d+\\|\\d+)" );
//
//        Matcher m = pattern.matcher( line );
//        Account account = configData.getActiveAccount();
//        int ticks = account.getTicks();
//        int galaxyTicks = account.getGalaxyTicks();
//        while( m.find() )
//        {
//            Coordinates coordinates = new Coordinates( m.group( 1 ) );
//            if( coordinates.isValid() )
//            {
//                Position position = coordinates.getPosition();
//                if( account != null )
//                {
//                    Planet myPlanet = getPlanetOnPosition( account, position );
//                    if( myPlanet != null )
//                    {
//                        myPlanet.setCoordinates( coordinates );
//                    }
//                    Map<Position, Coordinates> map = configData.getCoordinates( account );
//                    if( map !=  null )
//                    {
//                        // normalize (backtrack account ticks)
//                        map.put( position, coordinates.translate( -galaxyTicks, -ticks ) );
//                    }
//                }
//            }
//        }
//    }

    protected void actionSync() {
        try {
            Coordinator coordinator = new Coordinator(configData, false);
            setSync(coordinator.getCurrentGalaxyTicks(), coordinator.getCurrentTicks());
        }
        catch(IOException ex) {
            System.out.println(ex);
        }
    }

//    private Planet getPlanetOnPosition( Account account, Position position )
//    {
//        Planet planet = new Planet( null, position );
//        Map<Planet, Planet> lookupPlanets = account.getLookupPlanets();
//        Planet myPlanet = null;
//        if( lookupPlanets.containsKey( planet ) )
//        {
//            myPlanet = lookupPlanets.get( planet );
//        }
//        return myPlanet;
//    }


    public void update()
    {
        updateLabel();
        updateTable();
    }

    private void updateLabel()
    {
        labelGalaxyTicks.setText( configData.getI18nText( "Coordinator.galaxyTicks" ) );
        labelTicks.setText( configData.getI18nText( "Coordinator.ticks" ) );
        syncLabel.setText( "SYNC" );
        filterButton.setText( configData.getI18nText( "Coordinator.filter" ) );
        filterMyPlanets.setText( configData.getI18nText( "Coordinator.myPlanets" ) );
        mainFrame.getRootPane().setDefaultButton( filterButton );

        syncButton.setText( configData.getI18nText( "Coordinator.sync" ) );

//        importButton.setText( configData.getI18nText( "Planet.import" ) );
//        boolean isActive = ( configData.getActiveAccount() != null );
//        importField.setEnabled( isActive );
//        importField.setEditable( isActive );
//        importButton.setEnabled( isActive );

        Account account = configData.getActiveAccount();
        if( account != null )
        {
            galaxyTicks.setValue( account.getGalaxyTicks() );
            ticks.setValue( account.getTicks() );
            updateSync( account );
        }
    }

    private void updateSync( Account account )
    {
        if( account != null )
        {
            boolean sync = account.isSync();
            if( sync )
            {
                sync &= ( currentGalaxyTicks == account.getGalaxyTicks() ) && ( currentTicks == account.getTicks() );
            }
            syncLabel.setBackground( ( sync ) ? Color.green.darker() : Color.red.darker() );
//            importButton.setEnabled( sync );
        }
    }

    private void updateTable()
    {
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            currentTicks = current.getTicks();
            currentGalaxyTicks = current.getGalaxyTicks();
            planets.update();
            planets.setTicks( currentGalaxyTicks, currentTicks );
        }
        planets.fireTableStructureChanged();
        TableColumnModel columnModel = planetTable.getColumnModel();
        TableColumn column;
        int width;
        column = columnModel.getColumn( CoordinatorTableModel.ROW );
        width = 52;
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }

}
