/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.Parser;
import sitool.components.ListItem;
import sitool.components.Building;
import sitool.components.EnumWrapper;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Stage;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.BuildListTableModel;
import sitool.table.ImageTableCellRenderer;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.TooltipTable;
import sitool.technology.TechtreeDependency;

/**
 *
 * @author graf
 *
 */
public class BuildListPane extends JPanel implements ModulePane, ImportExportPane
{
    private static final long serialVersionUID = 7285176454851334424L;

    protected static final String extension = ".sbz";

    private MainFrame mainFrame;
    private ConfigData configData;
    private JComboBox<Planet> myPlanets = new JComboBox<>();
    private JCheckBox astro = new JCheckBox();
    private JCheckBox constructor = new JCheckBox();
    private JComboBox<EnumWrapper<Building>> buildings = new JComboBox<>();
    private JLabel planetLabel = new JLabel();
    private JLabel buildingLabel = new JLabel();
    private JLabel astroLabel = new JLabel();
    private JLabel constructorLabel = new JLabel();
    private JButton addButton = new JButton();
    private JButton deleteButton = new JButton();
    private JButton deleteAll = new JButton();
    private JButton importButton = new JButton();
    private JButton exportButton = new JButton();
    private JLabel canvas = new JLabel();
    private JPanel center;
    private BuildListTableModel model;
    private JTable table;
    private Account currentAccount;
    private TechtreeDependency techtreeDependency;

    public BuildListPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        currentAccount = configData.getActiveAccount();
        techtreeDependency = new TechtreeDependency( configData );

        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        Account account = configData.getActiveAccount();
        if( currentAccount == null || !currentAccount.equals( account ) )
        {
            model.removeAll();
        }
        currentAccount = account;
        if( account != null )
        {
            center.setVisible( true );
            updateMyPlanets();
            updateLabel();

            ItemEvent event;
            event = new ItemEvent( myPlanets, 0, myPlanets.getSelectedItem(), ItemEvent.SELECTED );
            actionPlanetChanged( event );
            initBuildings(astro.isSelected());
            event = new ItemEvent( buildings, 0, buildings.getSelectedItem(), ItemEvent.SELECTED );
            actionBuildingChanged( event );

            updateTable();
        }
        else
        {
            center.setVisible( false );
        }
    }

    private void updateLabel()
    {
        planetLabel.setText( configData.getI18nText( "Planet" ) );
        astroLabel.setText( configData.getI18nText( "Asteroid" ) );
        constructorLabel.setText( configData.getI18nText( "BuildList.constructor" ) );
        buildingLabel.setText( configData.getI18nText( "Building" ) );
        addButton.setText( configData.getI18nText( "add" ) );
        deleteButton.setText( configData.getI18nText( "delete" ) );
        deleteAll.setText( configData.getI18nText( "deleteAll" ) );
        importButton.setText( configData.getI18nText( "Load" ) );
        exportButton.setText( configData.getI18nText( "Save" ) );
    }

    private void updateTable()
    {
        model.fireTableStructureChanged();
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column;
        int width;
        column = table.getColumnModel().getColumn( 0 );
        width = 135;
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column = table.getColumnModel().getColumn( 1 );
        width = 170;
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column = columnModel.getColumn( 2 );
        width = 36;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( BuildListTableModel.TRANSMITTER );
        width = 55;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( BuildListTableModel.POINTS );
        width = 65;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = table.getColumnModel().getColumn( BuildListTableModel.DOWN );
        width = 16;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = table.getColumnModel().getColumn( BuildListTableModel.UP );
        width = 16;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
    }

    private void updateMyPlanets()
    {
        Object selection = myPlanets.getSelectedItem();
        myPlanets.removeAllItems();
        myPlanets.addItem( new Planet( "-- -- --", new Position( 0, 0, 0) ) );
        List<Planet> ownPlanets = new ArrayList<Planet>( configData.getMyPlanets() );
        for( Planet planet : ownPlanets )
        {
            myPlanets.addItem( planet );
        }
        // restore selection
        if( selection != null )
        {
            myPlanets.setSelectedItem( selection );
        }
        else
        {
            myPlanets.setSelectedIndex( 0 );
        }
        myPlanets.setMaximumRowCount( myPlanets.getItemCount() );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        initBuildings( false );
        deleteButton.setEnabled( false );

        JPanel north = new JPanel( new BorderLayout() );
        JPanel northwest = new JPanel( new GridBagLayout() );
        center.add( north, BorderLayout.NORTH );
        north.add( northwest, BorderLayout.WEST );
        north.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.insets = new Insets( 2, 2, 2, 2 );

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        northwest.add( planetLabel, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        northwest.add( astroLabel, constraints );
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        northwest.add( myPlanets, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        northwest.add( astro, constraints );
        constraints.gridx = 4;
        constraints.gridy = 0;
        constraints.gridwidth = 3;
        constraints.weightx = 0.0;
        northwest.add( buildingLabel, constraints );
        constraints.gridx = 7;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        northwest.add( constructorLabel, constraints );
        constraints.gridx = 4;
        constraints.gridy = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        northwest.add( buildings, constraints );
        constraints.gridx = 7;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        northwest.add( constructor, constraints );
        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        northwest.add( new JLabel( " " ), constraints );
        constraints.gridx = 3;
        constraints.gridy = 3;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        northwest.add( addButton, constraints );

        canvas.setPreferredSize( new Dimension( 120, 120 ) );
        canvas.setHorizontalAlignment( SwingConstants.CENTER );
        north.add( canvas, BorderLayout.CENTER );

        JPanel northeast = new JPanel( new FlowLayout() );
        JPanel buttonPanel = new JPanel( new GridLayout( 5, 1 ) );
        north.add( northeast, BorderLayout.EAST );
        northeast.add( buttonPanel );
        buttonPanel.add( deleteButton );
        buttonPanel.add( deleteAll );
        buttonPanel.add( Box.createVerticalGlue() );
        buttonPanel.add( importButton );
        buttonPanel.add( exportButton );

        model = new BuildListTableModel( mainFrame );
        final BlindTableCellRenderer summaryRenderer = new BlindTableCellRenderer();
        summaryRenderer.setHorizontalAlignment( SwingConstants.CENTER );
        final AlternateLineTableCellRenderer leftRenderer = new AlternateLineTableCellRenderer();
        final ResourceTableCellRenderer timeColumnRenderer = new ResourceTableCellRenderer();
        timeColumnRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        final ImageTableCellRenderer ic = new ImageTableCellRenderer();
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = -3394549916266421117L;

            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( row >= model.getRowCount() - 2 )
                {
                    rv = summaryRenderer;
                }
                else
                {
                    if( modelColumn < 2 )
                    {
                        rv = leftRenderer;
                    }
                    else if( modelColumn == BuildListTableModel.TIME )
                    {
                        rv = timeColumnRenderer;
                    }
                    else if( modelColumn == BuildListTableModel.DOWN ||
                                    modelColumn == BuildListTableModel.UP
                                    )
                    {
                        rv = ic;
                    }
                    else
                    {
                        rv = super.getCellRenderer( row, column );
                    }
                }
                return rv;
            }

        };
        table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        AlternateLineTableCellRenderer defaultRenderer = new AlternateLineTableCellRenderer();
        defaultRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        table.setDefaultRenderer( Object.class, defaultRenderer );
        table.addMouseListener( model );

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        myPlanets.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionPlanetChanged( e );
            }

        });
        buildings.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionBuildingChanged( e );
            }

        });
        astro.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isAstro = ( e.getStateChange() == ItemEvent.SELECTED );
                initBuildings( isAstro );
                addButton.setEnabled( enableAdd() );
            }
        });
        constructor.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isConstructor = ( e.getStateChange() == ItemEvent.SELECTED );
                model.setConstructor( isConstructor );
            }
        });
        addButton.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent e )
            {
                actionAdd();
            }
        });
        deleteButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionDelete();
            }

        });
        deleteAll.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                model.removeAll();
                Building building =  ((EnumWrapper<Building>)buildings.getSelectedItem()).getBase();
                if(building == Building.superComputer) {
                    addButton.setEnabled( enableAdd() );
                }
            }
        });
        importButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionLoad();
            }
        });
        exportButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSave();
            }
        });

        table.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    boolean mayDelete = false;
                    // single selection, only one index expected
                    int row = table.getSelectedRow();
                    // enable the button when selection is active (but don't delete the summary lines!)
                    mayDelete = ( row >= 0 && row < model.getRowCount() - 2 );
                    // check for dependencies
                    if( mayDelete && row < model.getRowCount() - 3 )
                    {
                        ListItem<Building> item2delete = model.getData().get( row );
                        // higher versions of same building
                        List<ListItem<Building>> data = model.getData().subList( row + 1, model.getRowCount() - 2 );
                        for( ListItem<Building> item : data )
                        {
                            if( item2delete.getPlanet().equals( item.getPlanet() ) && item2delete.isAstro() == item.isAstro() )
                            {
                                if( item2delete.getItem() == item.getItem() && item2delete.getLevel() < item.getLevel() )
                                {
                                    mayDelete = false;
                                    break;
                                }
                            }
                        }
                        // tech dependency
                        while( mayDelete && ++row < model.getRowCount() - 3 )
                        {
                            ListItem<Building> item2test = model.getData().get( row );
                            data = new ArrayList<ListItem<Building>>( model.getData().subList( 0, row ) );
                            data.remove( item2delete );
                            List<Planet> planets = new ArrayList<Planet>();
                            Planet planet = configData.getPlanet( item2test.getPlanet() );
                            Planet origin = model.copyPlanetAndUpdateFromModel( planet, item2test.isAstro(), data );
                            planets.add( origin );
                            techtreeDependency.setPlanets( planets );
                            mayDelete = techtreeDependency.isEnabled( item2test.getItem() );
                        }
                    }
                    deleteButton.setEnabled( mayDelete );
                }
            }
        });
        table.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "delete");
        table.getActionMap().put("delete", new AbstractAction() {
            private static final long serialVersionUID = 5999113634471333070L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                if(deleteButton.isEnabled()) {
                    actionDelete();
                }
            }
        });

        return center;
    }

    @SuppressWarnings("unchecked")
    protected void actionAdd()
    {
        Planet planet = (Planet)myPlanets.getSelectedItem();
        Building building =  ((EnumWrapper<Building>)buildings.getSelectedItem()).getBase();
        boolean isAstro = astro.isSelected();
        String name = ( isAstro ) ? configData.getI18nText( "Asteroid" ) : planet.getName();
        Stage stage = planet.getStage( building, isAstro );
        int level = ( stage != null ) ? stage.getLevel() + 1 : 1;
        Planet origin = new Planet( name, planet.getPosition() );
        ListItem<Building> item = new ListItem<>( origin, isAstro, building, level );
        // adjust level (if necessary)
        List<ListItem<Building>> data = model.getData();
        while( data.contains( item  ) )
        {
            item.increaseLevel();
        }
        model.addItem( item );
        // update techtreeDependency
        stage = techtreeDependency.getPlanets().get( 0 ).getStage( building, false );
        if( stage == null )
        {
            stage = new Stage(building, 0);
            techtreeDependency.getPlanets().get( 0 ).addBuilding( building, stage, false );
        }
        stage.setLevel( item.getLevel() );
        // super computer dependency update
        if(building == Building.superComputer) {
            addButton.setEnabled( enableAdd() );
        }
    }

    protected void actionDelete()
    {
        int row = table.getSelectedRow();
        model.removeRow( row );
        Building building =  ((EnumWrapper<Building>)buildings.getSelectedItem()).getBase();
        if(building == Building.superComputer) {
            addButton.setEnabled( enableAdd() );
        }
    }

    protected void actionPlanetChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            astro.setSelected( false );
            addButton.setEnabled( enableAdd() );
        }
    }

    @SuppressWarnings("unchecked")
    protected void actionBuildingChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Building building = ((EnumWrapper<Building>)e.getItem()).getBase();

            canvas.setIcon( building.getIcon() );
            canvas.repaint();
            addButton.setEnabled( enableAdd() );
        }
    }

    protected void actionLoad()
    {
        Document doc = configData.readData( this );
        if( doc != null )
        {
            NodeList accounts = doc.getElementsByTagName( Parser.Tag_Account );
            if( accounts.getLength() > 0 )
            {
                Element account = (Element)accounts.item( 0 );
                String accountId = account.getAttribute( Parser.Attribute_Name );
                // load for current account only
                if( currentAccount != null && currentAccount.getId().equals( accountId ) )
                {
                    List<ListItem<Building>> data = new ArrayList<ListItem<Building>>();
                    NodeList buildList = account.getElementsByTagName( Parser.Tag_BuildlistItem );
                    if( buildList.getLength() > 0 )
                    {
                        String asteroid = configData.getI18nText( "Asteroid" );
                        ListItem<Building>[] items = new ListItem[buildList.getLength()];
                        int i;
                        // build build list data
                        for( i = 0; i < items.length; i++ )
                        {
                            Element itemNode = ( Element )buildList.item( i );
                            String name = itemNode.getAttribute( Parser.Attribute_Name );
                            boolean isAstro = asteroid.equals( name );
                            int galaxy = ConfigData.parseIntegerAttribute( itemNode.getAttribute( Parser.Attribute_Galaxy ) );
                            int system = ConfigData.parseIntegerAttribute( itemNode.getAttribute( Parser.Attribute_System ) );
                            int p = ConfigData.parseIntegerAttribute( itemNode.getAttribute( Parser.Attribute_Planet ) );
                            Planet planet = new Planet( name, new Position( galaxy, system, p ) );
                            // look if we still have a planet on this position, the name might have changed
                            Planet myPlanet = configData.getPlanet( planet );
                            if( myPlanet != null )
                            {
                                // check astro
                                if( isAstro )
                                {
                                    if( myPlanet.getAstroSize() == 0 )
                                    {
                                        planet = null;
                                    }
                                }
                                else
                                {
                                    // name may have changed
                                    planet.setName( myPlanet.getName() );
                                }
                            }
                            else
                            {
                                planet = null;
                            }
                            if( planet != null )
                            {
                                Building building = Building.valueOf( itemNode.getAttribute( Parser.Attribute_Building ) );
                                int level = ConfigData.parseIntegerAttribute( itemNode.getAttribute( Parser.Attribute_Level ) );
                                // check against existing buildings
                                Stage stage = myPlanet.getStage(building, isAstro);
                                if( stage == null || stage.getLevel() < level )
                                {
                                    int position = ConfigData.parseIntegerAttribute( itemNode.getAttribute( Parser.Attribute_Position ) );
                                    items[position] = new ListItem<Building>( planet, isAstro, building, level );
                                }
                            }
                        }
                        for( i = 0; i < items.length; i++ )
                        {
                            // add all non-null items
                            if( items[i] != null )
                            {
                                data.add( items[i] );
                            }
                        }
                    }
                    model.setData( data );
                }
            }
        }
    }

    protected void actionSave()
    {
        configData.saveData( this );
    }

    private void initBuildings( boolean isAstro )
    {
        Object item = buildings.getSelectedItem();
        buildings.removeAllItems();
        Building[] b = ( isAstro ) ? Building.astroBuildings() : Building.planetBuildings(configData.getVersion());
        for( Building building : b )
        {
            buildings.addItem( new EnumWrapper<Building>( configData, building ) );
        }
        buildings.setSelectedIndex(0);
        if(item != null) {
            buildings.setSelectedItem(item);
        }
        buildings.setMaximumRowCount( b.length );
    }

    @SuppressWarnings("unchecked")
    private boolean enableAdd()
    {
        boolean rv = false;
        Planet planet = ( Planet )myPlanets.getSelectedItem();
        rv = planet.getPosition().isRealPosition();
        boolean isAstro = astro.isSelected();
        if( rv && isAstro )
        {
            rv = ( planet.getAstroSize() > 0 );
        }
        if( rv )
        {
            List<Planet> planets = new ArrayList<Planet>();
            Planet origin = model.copyPlanetAndUpdateFromModel( planet, isAstro, model.getData() );
            planets.add( origin );
            techtreeDependency.setPlanets( planets );
            Building building = ((EnumWrapper<Building>)buildings.getSelectedItem()).getBase();
            rv = techtreeDependency.isEnabled( building );
            // special super computer check
            if( rv && building == Building.superComputer) {
                Stage stage = planet.getStage( building, isAstro );
                int level = ( stage != null ) ? stage.getLevel() + 1 : 1;
                ListItem<Building> item = new ListItem<>( origin, isAstro, building, level );
                // adjust level (if necessary)
                List<ListItem<Building>> data = model.getData();
                while( data.contains( item  ) )
                {
                    item.increaseLevel();;
                }
                Stage lab = planet.getStage( Building.researchLab, false );
                int labLevel = ( lab != null ) ? lab.getLevel() : 0;
                rv = (labLevel - 13 > item.getLevel());
            }
        }
        return rv;
    }

    public Document getData(Parser xmlParser)
    {
        Document doc = null;
        if( xmlParser != null && currentAccount != null )
        {
            doc = xmlParser.getDocument();
            Element rootEl = doc.createElement( Parser.Tag_Account );
            doc.appendChild( rootEl );
            rootEl.setAttribute( Parser.Attribute_Name, currentAccount.getId() );
            int position = 0;
            for(ListItem<Building> item : model.getData() )
            {
                Element buildListNode = doc.createElement( Parser.Tag_BuildlistItem );
                rootEl.appendChild( buildListNode );
                buildListNode.setAttribute( Parser.Attribute_Position, Integer.toString( position ) );
                buildListNode.setAttribute( Parser.Attribute_Building, item.getItem().name() );
                Position planetPosition = item.getPlanet().getPosition();
                buildListNode.setAttribute( Parser.Attribute_Name, item.getPlanet().getName() );
                buildListNode.setAttribute( Parser.Attribute_Galaxy, Integer.toString( planetPosition.getGalaxy() ) );
                buildListNode.setAttribute( Parser.Attribute_System, Integer.toString( planetPosition.getSystem() ) );
                buildListNode.setAttribute( Parser.Attribute_Planet, Integer.toString( planetPosition.getPlanet() ) );
                buildListNode.setAttribute( Parser.Attribute_Level, Integer.toString( item.getLevel() ) );
                position++;
            }
        }
        return doc;
    }

    public String getExtension()
    {
        return extension;
    }

    public String getFileName()
    {
        return configData.getI18nText( "BuildList.title" );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    public File getCurrentDir()
    {
        return configData.getAccountDir();
    }

    public void setCurrentDir( File currentDir )
    {
        configData.setAccountDir( currentDir );
    }
}
