/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.TotalPointSource;
import sitool.table.BlindTableCellRenderer;
import sitool.table.BuildingTableCellRenderer;
import sitool.table.PlanetTableHeaderMouseListener;
import sitool.table.BuildingTableModel;
import sitool.table.MultiLineTableCellHeaderRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.SpinnerCellEditor;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class BuildingPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private MainFrame mainFrame;
    private JPanel center;
    private JTable table;
    private BuildingTableModel model;

    public BuildingPane( MainFrame parent )
    {
        mainFrame = parent;
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        if( mainFrame.getConfigData().getActiveAccount() != null )
        {
            center.setVisible( true );
            model.update();
            model.fireTableStructureChanged();

            // define column width and row height
            int rowHeight = new JComboBox<>().getPreferredSize().height - 3;
            table.setRowHeight( rowHeight );
            // fix the first column
            int width = 180;
            TableColumn column = table.getColumnModel().getColumn( 0 );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
        }
        else
        {
            center.setVisible( false );
        }
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        final TableCellRenderer headerRenderer = new BlindTableCellRenderer();
        final TableCellRenderer rowRenderer = new BuildingTableCellRenderer();
        JScrollPane scrollPane;
        model = new BuildingTableModel( mainFrame );
        mainFrame.registerTotalPointSource( TotalPointSource.Type.Buildings, model );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = 7149497989459592130L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 )
                {
                    rv = headerRenderer;
                }
                else if( row == BuildingTableModel.planetLine
                      || row == model.getAstroLine()
                      || row == model.getRows() )
                {
                    rv = rowRenderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }
        };
        table.addMouseListener( model );
        JTableHeader header = table.getTableHeader();
        header.addMouseListener( new PlanetTableHeaderMouseListener( PlanetTableHeaderMouseListener.Type.Building ) );
        TableCellRenderer renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table.setDefaultRenderer( Number.class, new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) ) );
        TableCellEditor editor = new SpinnerCellEditor( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        editor.addCellEditorListener( model );
        table.setDefaultEditor( Number.class, editor );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_NEVER,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );

        return center;
    }

}
