/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.w3c.dom.Document;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.Parser;
import sitool.components.ButtonTabbedPane;
import sitool.components.ModulePane;

/**
 * @author graf
 *
 */
public class ResourceSummaryPane extends JPanel implements ModulePane, ImportExportPane
{
    private static final long serialVersionUID = -4027491488562178409L;
    protected static final String extension = ".stz";
    private File currentDir;

    private MainFrame mainFrame = null;
    private ConfigData configData = null;

    private JTabbedPane pane = new ButtonTabbedPane(ConfigData.getIcon( "/close.png" ), ConfigData.getIcon( "/save.png" ));
    private JPanel center;
    private static SimpleDateFormat titleFormat = new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" );
    private static SimpleDateFormat fileFormat = new SimpleDateFormat( "yyyyMMddHHmm" );

    private JButton buttonNew = new JButton();
    private JButton buttonLoad = new JButton();

    private JLabel labelMultipleProduction = new JLabel();
    private JSpinner spinnerMultipleProduction;
    private JLabel labelProductionBonus = new JLabel();
    private JSpinner spinnerProductionBonus;
    private JLabel labelAllianceSpecialization = new JLabel();
    private JSpinner spinnerAllianceSpecialization;

    public ResourceSummaryPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();

        setLayout( new BorderLayout() );
        setBorder( null );

        // try application dir
        String configDir = System.getProperty( "user.dir" );
        currentDir = new File( configDir );
        center = createCenterPanel();
        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        if( mainFrame.getConfigData().getActiveAccount() != null )
        {
            updateLabel();
            center.setVisible( true );
            // propagate update
            setMultiplier();
            Component c = pane.getSelectedComponent();
            if( c instanceof ModulePane )
            {
                ((ModulePane)c).update();
                //System.out.println( "update pane " + c.getClass().getSimpleName() );
            }
        }
        else
        {
            center.setVisible( false );
        }
    }

    private void updateLabel()
    {
        buttonNew.setText( configData.getI18nText( "New" ) );
        buttonLoad.setText( configData.getI18nText( "Load" ) );
        labelMultipleProduction.setText( configData.getI18nText( "Resources.multipleProduction" ) );
        labelProductionBonus.setText( configData.getI18nText( "Resources.mentorBonus" ) );
        labelAllianceSpecialization.setText( configData.getI18nText( "Resources.allianceSpecialization.producer" ) );
    }

    public JTabbedPane getTabbedPane()
    {
        return pane;
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );

        JPanel upper = new JPanel( new BorderLayout() );
        upper.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        JPanel left = new JPanel( new FlowLayout(FlowLayout.LEFT, 12, 4) );
        JPanel buttonPanel = new JPanel( new GridLayout( 1, 3, 12, 4 ) );
        buttonPanel.add( buttonNew );
        buttonNew.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionNew();
            }

        });
        //defaultButton = button;
        buttonLoad.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionLoad();
            }

        });
        buttonPanel.add( buttonLoad );
        left.add( buttonPanel );
        left.add( labelMultipleProduction );
        spinnerMultipleProduction = new JSpinner(new SpinnerNumberModel( 1, 1, 99999, 1 ));
        spinnerMultipleProduction.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                setMultiplier();
            }
        });
        left.add(spinnerMultipleProduction);

        left.add( labelProductionBonus );
        spinnerProductionBonus = new JSpinner(new SpinnerNumberModel( 0, 0, 100, 1 ));
        spinnerProductionBonus.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                setMultiplier();
            }
        });
        left.add(spinnerProductionBonus);

        left.add( labelAllianceSpecialization );
        spinnerAllianceSpecialization = new JSpinner(new SpinnerNumberModel( 0, 0, 100, 1 ));
        spinnerAllianceSpecialization.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                setMultiplier();
            }
        });
        left.add(spinnerAllianceSpecialization);

        upper.add( left, BorderLayout.WEST );
        panel.add( upper, BorderLayout.NORTH );

        //actionNew();

        pane.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                update();
            }
        });
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

    protected void setMultiplier() {
        double multipleProduction = ((Number)spinnerMultipleProduction.getValue()).doubleValue();
        int productionBonus = ((Number)spinnerProductionBonus.getValue()).intValue();
        int allianceSpecialization = ((Number)spinnerAllianceSpecialization.getValue()).intValue();
        for(int index = 0; index < pane.getTabCount(); index++) {
            Component c = pane.getComponentAt(index);
            if(c instanceof ResourceModulePane) {
                ((ResourceModulePane)c).setMultipleProduction(multipleProduction, productionBonus, allianceSpecialization);
            }
        }
    }

    protected void actionNew()
    {
        addTab( new Date() );
    }

    protected void actionLoad()
    {
        addTab( configData.readData( this ), null );
    }

    public Document getData( Parser xmlParser )
    {
        Document doc = null;
        if( xmlParser != null )
        {
            doc = xmlParser.getDocument();

            // delegate
            ResourcePane child = ( ResourcePane )pane.getSelectedComponent();
            child.getResourceData( doc );
        }
        return doc;
    }

    public String getTitle(Date date)
    {
        return titleFormat.format( date ) ;
    }

    public String getFileName()
    {
        return getFileName(pane.getSelectedComponent());
    }

    protected String getFileName(Component component)
    {
        int index = pane.indexOfComponent(component);
        Date date;
        try
        {
            date = titleFormat.parse( pane.getTitleAt( index ) );
        }
        catch( ParseException e )
        {
            date = new Date();
        }
        StringBuilder sb = new StringBuilder();
        sb.append(  fileFormat.format( date ) );
        sb.append('-').append(configData.getActiveAccount().getId());
        return sb.toString();
    }

    public String getExtension()
    {
        return extension;
    }

    public File getCurrentDir()
    {
        return currentDir;
    }

    public void setCurrentDir( File currentDir )
    {
        this.currentDir = currentDir;
    }

    public ResourcePane addTab(Date date)
    {
        return addTab(null, date);
    }

    /**
     * @param doc
     * @param date
     * @return
     */
    private ResourcePane addTab(Document doc, Date date)
    {
        ResourcePane resourcePane = new ResourcePane( this );
        String title = ( doc != null) ? resourcePane.setResourceData( doc ) : getTitle(date);
        pane.addTab( title, resourcePane );
        update();
        return resourcePane;
    }
}
