/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

import sitool.MainFrame;
import sitool.components.DateTimeField;


/**
 * 
 * @author graf
 *
 */
public class DateBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -3286773837495679101L;

    private SaveCalcPane parent;
    private MainFrame mainFrame = null;
    private DateTimeField field = null;
    
	public DateBox( SaveCalcPane parent )
    {
        super( parent.getMainFrame() );
        this.parent = parent;
        mainFrame = parent.getMainFrame();

      	enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( mainFrame.getConfigData().getI18nText( "DateBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        
        JPanel center = createCenterPanel();
        
        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( mainFrame.getConfigData().getI18nText( "OK" ) );;
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );
        
        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new GridLayout( 2, 1 ) );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JLabel label = new JLabel( mainFrame.getConfigData().getI18nText( "DateBox.label" ) );
        center.add( label );
        field = new DateTimeField();
        center.add( field );
        
        return center;
    }
    
    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        Date date = field.getDate();
        long duration = date.getTime() - System.currentTimeMillis();
        duration = ( duration > 0 ) ? duration / 2000L + 31 : 0; 
        parent.setDuration( duration );
        dispose();
    }
}
