/****************************************
 *  COPYRIGHT (C) 2006
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.FleetSpeed;
import sitool.components.Research;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.TechValue;
import sitool.components.Technology;
import sitool.components.Version;

/**
 *
 * @author graf
 *
 */
public class FleetSpeedBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    protected static final Color off = UIManager.getColor("control");
    protected static final Color on = Color.black;
    protected static final Color lowest = Color.red.brighter();
    private static Pattern prepTimePattern = Pattern.compile( "((\\d+):)?(\\d+):(\\d+)" );

    private ConfigData configData = null;
    private Version version;
    private final List<TechValue> techValues = new ArrayList<TechValue>();

    private List<Ship> allShips;
    private JCheckBox[] checks;
    private JLabel[][] tips;
    private FleetSpeed lowestSpeed = new FleetSpeed();

    private JSpinner rocketTech;
    private JSpinner diffusionTech;
    private JSpinner warpTech;
    private JSpinner speedBonus;

    public FleetSpeedBox( MainFrame parent )
    {
        super( parent );

        configData = parent.getConfigData();
        version = configData.getVersion();
        for( int i = 0; i < 51; i++ )
        {
            techValues.add( new TechValue( configData, i ) );
        }

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "FleetspeedBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JPanel northPanel = new JPanel( new GridLayout( 4, 2 ) );
        northPanel.add( new JLabel( configData.getI18nText( Research.rocket.name() ) ) );
        ChangeListener cl = new ChangeListener() {

            public void stateChanged( ChangeEvent e )
            {
                actionTechChanged();
            }

        };

        rocketTech = new JSpinner( new SpinnerListModel( techValues ) );
        northPanel.add( rocketTech );
        northPanel.add( new JLabel( configData.getI18nText( Research.diffusion.name() ) ) );
        diffusionTech = new JSpinner( new SpinnerListModel( techValues ) );
        northPanel.add( diffusionTech );
        northPanel.add( new JLabel( configData.getI18nText( Research.warp.name() ) ) );
        warpTech = new JSpinner( new SpinnerListModel( techValues ) );
        northPanel.add( warpTech );
        speedBonus = new JSpinner(new SpinnerNumberModel( 0, 0, 4, 1 ));
        northPanel.add( new JLabel( configData.getI18nText( "Account.fleeter" ) ) );
        northPanel.add(speedBonus);
        center.add( northPanel, BorderLayout.NORTH );
        // set initial values
        Technology speedTech = configData.getMyTech();
        if( speedTech != null )
        {
            TechValue techValue = new TechValue( configData, speedTech.getRocket() );
            rocketTech.getModel().setValue( techValue );
            techValue = new TechValue( configData, speedTech.getDiffusion() );
            diffusionTech.getModel().setValue( techValue );
            techValue = new TechValue( configData, speedTech.getWarp() );
            warpTech.getModel().setValue( techValue );
            speedBonus.getModel().setValue(Integer.valueOf(configData.getSpecialization().getFleeter()));
        }

        // now add listeners
        rocketTech.addChangeListener( cl );
        diffusionTech.addChangeListener( cl );
        warpTech.addChangeListener( cl );
        speedBonus.addChangeListener( cl );

        allShips = Arrays.asList( Ship.movableShips( configData.getVersion() ) );
        checks = new JCheckBox[allShips.size()];
        tips = new JLabel[checks.length][3];

        JPanel southPanel = new JPanel( new BorderLayout() );
        JPanel checkPanel = new JPanel( new GridLayout( checks.length, 1, 15, 5 ) );
        JPanel labelPanel = new JPanel( new GridLayout( checks.length, 3, 15, 5 ) );
        southPanel.add( checkPanel, BorderLayout.WEST );
        southPanel.add( labelPanel, BorderLayout.CENTER );
        for( int i = 0; i < checks.length; i++ )
        {
            Ship ship = allShips.get( i );
            checks[i] = new JCheckBox( configData.getI18nText( ship.name() ) );
            checks[i].setSelected( false );
            checks[i].addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent e )
                {
                    actionShipChecked();
                }
            });
            checkPanel.add( checks[i] );
            tips[i][0] = new JLabel();
            tips[i][1] = new JLabel();
            tips[i][2] = new JLabel();
            for( int k = 0; k < tips[i].length; k++ )
            {
                tips[i][k].setForeground( off );
                tips[i][k].setHorizontalAlignment( SwingConstants.RIGHT );
                labelPanel.add( tips[i][k] );
            }
        }
        paintValues();
        center.add( southPanel, BorderLayout.CENTER );
        return center;
    }


    private TechValue getTechValue( Ship.Type type )
    {
        TechValue techValue = null;
        switch( type )
        {
            case Rocketdrive:
                techValue = ( TechValue )rocketTech.getValue();
                break;
            case Diffusiondrive:
                techValue = ( TechValue )diffusionTech.getValue();
                break;
            case Warpdrive:
                techValue = ( TechValue )warpTech.getValue();
                break;
        }
        return techValue;
    }

    private void paintValues()
    {
        for( int i = 0; i < checks.length; i++ )
        {
            Ship ship = allShips.get( i );
            Specialization specialization = new Specialization();
            specialization.setFleeter(((Number)speedBonus.getValue()).intValue());
            TechValue techValue = getTechValue( ship.getType() );
            long velocity;
            if( version != Version.v20 )
            {
                velocity = (100 + specialization.getSpeedbonus(ship) + 10 * techValue.getValue() ) * ship.getVelocity() / 100;
                tips[i][0].setText( "" );
                tips[i][1].setText( "" );
                tips[i][2].setText( Long.toString( velocity ) );
            }
            else
            {
                int acceleration = (10 + techValue.getValue() ) * ship.getAcceleration() / 10;
                velocity = (20 + techValue.getValue() ) * ship.getVelocity2() / 20;
                tips[i][0].setText( ConfigData.formatTime( ship.getPreparationTime() ) );
                tips[i][1].setText( Integer.toString( acceleration ) );
                tips[i][2].setText( Long.toString( velocity ) );
            }
        }
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        if( lowestSpeed.getVelocity() != Long.MAX_VALUE )
        {
            configData.setVelocity( lowestSpeed );
        }
        dispose();
    }

    protected void actionShipChecked()
    {
        // brute force;
        lowestSpeed.reset();
        for( int i = 0; i < allShips.size(); i++ )
        {
            if( checks[i].isSelected() )
            {
                for( int k = 0; k < tips[i].length; k++ )
                {
                    tips[i][k].setForeground( on );
                }
                long velocity = Long.valueOf( tips[i][2].getText() );
                if( velocity < lowestSpeed.getVelocity() )
                {
                    lowestSpeed.setVelocity( velocity );
                    lowestSpeed.getShips()[FleetSpeed.VELOCITY] = allShips.get( i );
                }
                if( version == Version.v20 )
                {
                    int prepTime = parsePrepTime( tips[i][0].getText() );
                    if( prepTime > lowestSpeed.getPreptime() )
                    {
                        lowestSpeed.setPreptime( prepTime );
                        lowestSpeed.getShips()[FleetSpeed.PREPERATIONTIME] = allShips.get( i );
                    }
                    int acceleration = Integer.valueOf( tips[i][1].getText() );
                    if( acceleration < lowestSpeed.getAcceleration() )
                    {
                        lowestSpeed.setAcceleration( acceleration );
                        lowestSpeed.getShips()[FleetSpeed.ACCELERATION] = allShips.get( i );
                    }
                }
            }
            else
            {
                for( int k = 0; k < tips[i].length; k++ )
                {
                    tips[i][k].setForeground( off );
                }
            }
        }
        if( lowestSpeed.getVelocity() != Long.MAX_VALUE )
        {
            for( int i = 0; i < tips.length; i++ )
            {
                if( checks[i].isSelected() )
                {
                    if( version == Version.v20 )
                    {
                        if( ConfigData.formatTime( lowestSpeed.getPreptime() ).equals( tips[i][0].getText() ) )
                        {
                            tips[i][0].setForeground( lowest );
                        }
                        if( Integer.toString( lowestSpeed.getAcceleration() ).equals( tips[i][1].getText() ) )
                        {
                            tips[i][1].setForeground( lowest );
                        }
                    }
                    if( Long.toString( lowestSpeed.getVelocity() ).equals( tips[i][2].getText() ) )
                    {
                        tips[i][2].setForeground( lowest );
                    }
                }
            }
        }
        repaint();
    }

    protected void actionTechChanged()
    {
        // repaint lowest speed
        paintValues();
        actionShipChecked();
    }

    private int parsePrepTime( String text )
    {
        int rv = 0;
        Matcher matcher = prepTimePattern.matcher( text  );
        if( matcher.find() )
        {
            String hour = matcher.group( 1 );
            int h = ( hour != null ) ? Integer.parseInt( hour ) : 0;
            int m = Integer.parseInt( matcher.group( 3 ) );
            int s = Integer.parseInt( matcher.group( 4 ) );
            rv = Calculator.secondsPerHour * h + Calculator.secondsPerMinute * m + s;
        }
        return rv;
    }
}
