/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class FlightTimePane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 2758582246555940481L;
    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JTabbedPane pane;

    private Map<String, Integer> titles = new HashMap<String, Integer>();

    public FlightTimePane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( null );

        add( createCenterPanel(), BorderLayout.CENTER );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    public void update()
    {
        // propagate update
        int selection = pane.getSelectedIndex();
        Component c = pane.getSelectedComponent();
        if( c instanceof ModulePane )
        {
            ((ModulePane)c).update();
            //System.out.println( "update pane " + c.getClass().getSimpleName() );
        }
        for( Map.Entry<String, Integer> entry : titles.entrySet() )
        {
            int index = entry.getValue().intValue();
            pane.setTitleAt( index, configData.getI18nText( entry.getKey() ) );
            boolean enabled = true;
            if( entry.getKey() == "Savecalc.table.speed" )
            {
                enabled = configData.getVersion() != Version.v20;
                pane.setEnabledAt( index, enabled );
            }

            if( !enabled && index == selection )
            {
                pane.setSelectedIndex( 0 );
            }
        }
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );

        String key;
        int index = 0;

        pane = new JTabbedPane( JTabbedPane.TOP );
        FlightDurationPane duration = new FlightDurationPane( mainFrame );
        key = "Flighttime.duration";
        pane.add( duration, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        FlightVelocityPane velocity = new FlightVelocityPane( mainFrame );
        key = "Savecalc.table.speed";
        pane.add( velocity, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        pane.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                update();
            }
        });
        pane.setSelectedIndex( 0 );
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

}
