/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.statistic;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.EnumWrapper;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Stage;
import sitool.table.BuildingStatisticsTableModel;
import sitool.table.MultiLineTableCellHeaderRenderer;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class BuildingStatisticsPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -7866132208680948821L;

    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JComboBox<Planet> myPlanets = new JComboBox<>();
    private JCheckBox astro = new JCheckBox();
    private JSpinner temperature = new JSpinner();
    private JSpinner develC = new JSpinner();
    private JSpinner msa = new JSpinner();
    private JCheckBox constructor = new JCheckBox();
    private JComboBox<EnumWrapper<Building>> buildings = new JComboBox<>();
    private JSpinner level = new JSpinner();
    private JLabel canvas = new JLabel();
    private JPanel center;
    private JTable table;
    private BuildingStatisticsTableModel model;

    private JLabel planetLabel = new JLabel();
    private JLabel astroLabel = new JLabel();
    private JLabel devCLabel = new JLabel();
    private JLabel msaLabel = new JLabel();
    private JLabel constructorLabel = new JLabel();
    private JLabel tempLabel = new JLabel();
    private JLabel buildingLabel = new JLabel();
    private JLabel maxLevelLabel = new JLabel();

    private boolean inPlanetUpdate = false;
    private Object selectedPlanet = null;
    private Object selectedBuilding = null;

    public BuildingStatisticsPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        canvas.setPreferredSize( new Dimension( 120, 120 ) );
        canvas.setHorizontalAlignment( SwingConstants.CENTER );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateLabel();
        updateMyPlanets();
        updateTable();
    }

    private void updateLabel()
    {
        planetLabel.setText( configData.getI18nText( "Planet" ) );
        astroLabel.setText( configData.getI18nText( "Asteroid" ) );
        devCLabel.setText( configData.getI18nText( "DevC.level" ) );
        msaLabel.setText( configData.getI18nText( "Msa.level" ) );
        constructorLabel.setText( configData.getI18nText( "BuildList.constructor" ) );
        tempLabel.setText( configData.getI18nText( "Temperature" ) );
        buildingLabel.setText( configData.getI18nText( "Building" ) );
        maxLevelLabel.setText( configData.getI18nText( "max.Level" ) );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        level.setModel( new SpinnerNumberModel( 35, 1, 150, 1 ) );
        temperature.setModel( new SpinnerNumberModel( 0, -500, 500, 1 ) );
        develC.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );
        msa.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );
        initBuildings( false );

        JPanel west = new JPanel( new GridBagLayout() );
        center.add( west, BorderLayout.WEST );
        west.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.insets = new Insets( 2, 2, 2, 2 );

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( planetLabel, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astroLabel, constraints );
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( myPlanets, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astro, constraints );

        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( tempLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( temperature, constraints );
        constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( devCLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( develC, constraints );
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( msaLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( msa, constraints );

        constraints.gridx = 0;
        constraints.gridy = 5;
        constraints.gridwidth = 2;
        constraints.weightx = 0.0;
        west.add( constructorLabel, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 2;
        constraints.weightx = 0.0;
        west.add( constructor, constraints );

        constraints.gridx = 0;
        constraints.gridy = 6;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( buildingLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( buildings, constraints );
        constraints.gridx = 0;
        constraints.gridy = 7;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( maxLevelLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( level, constraints );
        constraints.gridx = 0;
        constraints.gridy = 8;
        constraints.gridwidth = 4;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.gridheight = GridBagConstraints.REMAINDER;
        west.add( canvas, constraints );

        model = new BuildingStatisticsTableModel( mainFrame );
        table = new TooltipTable( model );
        JTableHeader header = table.getTableHeader();
        TableCellRenderer renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table.setDefaultRenderer( Object.class, new ResourceTableCellRenderer() );

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        myPlanets.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionPlanetChanged( e );
            }

        });
        buildings.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionBuildingChanged( e );
            }

        });
        level.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                actionLevelChanged( e );
            }

        });
        develC.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setDevCenterLevel( level );
                adjustColumns();
            }

        });
        msa.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setMsaLevel( level );
                adjustColumns();
            }

        });
        temperature.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int temp = ((Number)spinner.getValue()).intValue();
                model.setTemperature( temp );
                adjustColumns();
            }

        });
        astro.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isAstro = ( e.getStateChange() == ItemEvent.SELECTED );
                Planet planet = ( Planet )myPlanets.getSelectedItem();
                int level;
                level = ( isAstro ) ? 0 : planet.getTemperature(configData.getVersion());
                temperature.setValue( Integer.valueOf( level ) );
                Stage stage;
                stage = planet.getStage( Building.developmentCenter, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                develC.setValue( level );
                stage = planet.getStage( Building.msa, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                msa.setValue( level );
                model.setAstro( isAstro );
                initBuildings( isAstro );
            }
        });
        constructor.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isConstructor = ( e.getStateChange() == ItemEvent.SELECTED );
                model.setConstructor( isConstructor );
            }
        });

        return center;
    }

    private void updateTable()
    {
        model.fireTableStructureChanged();
        ItemEvent event;
        event = new ItemEvent( myPlanets, 0, myPlanets.getSelectedItem(), ItemEvent.SELECTED );
        actionPlanetChanged( event );
        event = new ItemEvent( buildings, 0, buildings.getSelectedItem(), ItemEvent.SELECTED );
        actionBuildingChanged( event );
        ChangeEvent e = new ChangeEvent( level );
        actionLevelChanged( e );
    }

    private void updateMyPlanets()
    {
        Object selection = myPlanets.getSelectedItem();
        inPlanetUpdate = true;
        myPlanets.removeAllItems();
        myPlanets.addItem( new Planet( "-- -- --", new Position( 0, 0, 0) ) );
        List<Planet> ownPlanets = new ArrayList<Planet>( configData.getMyPlanets() );
        for( Planet planet : ownPlanets )
        {
            myPlanets.addItem( planet );
        }
        // restore selection
        if( selection != null )
        {
            myPlanets.setSelectedItem( selection );
        }
        else
        {
            myPlanets.setSelectedIndex( 0 );
        }
        myPlanets.setMaximumRowCount( myPlanets.getItemCount() );
        inPlanetUpdate = false;
        selectedPlanet = myPlanets.getSelectedItem();
    }

    protected void actionPlanetChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED && !inPlanetUpdate && !e.getItem().equals( selectedPlanet ) )
        {
            Planet planet = ( Planet )e.getItem();
            astro.setSelected( false );
            temperature.setValue( Integer.valueOf( planet.getTemperature(configData.getVersion()) ) );
            int level;
            Stage stage;
            stage = planet.getStage( Building.developmentCenter, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            develC.setValue( level );
            stage = planet.getStage( Building.msa, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            msa.setValue( level );
            model.setPlanet( planet );
            selectedPlanet = planet;
        }
    }

    @SuppressWarnings("unchecked")
    protected void actionBuildingChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED && !e.getItem().equals( selectedBuilding ) )
        {
            Building building = ((EnumWrapper<Building>)e.getItem()).getBase();
            selectedBuilding = e.getItem();
            model.setBuilding( building );

            int l = ((Number)level.getValue()).intValue();
            // set some useful limits
            switch( building )
            {
                case developmentCenter:
                    l = 12;
                    break;
                case weaponsFactory:
                    l = 13;
                    break;
                case particleCannon:
                case particleShield:
                case teleporter:
                case msa:
                    l = 15;
                    break;
                case ironStorage:
                case metalStorage:
                case kryptoniteStorage:
                case spiceStorage:
                    l = 21;
                    break;
                case researchLab:
                    l = 25;
                    break;
                case fleetBase:
                case scanner:
                    l = 20;
                    break;
                case superComputer:
                    l = 10;
                    break;
                default:
                    l = 40;
            }
            level.setValue( Integer.valueOf( l ) );

            canvas.setIcon( building.getIcon() );
            canvas.repaint();
            adjustColumns();
        }
    }

    protected void actionLevelChanged( ChangeEvent e )
    {
        JSpinner spinner = ( JSpinner )e.getSource();
        int level = ((Number)spinner.getValue()).intValue();
        model.setMaxRows( level );
        adjustColumns();
    }

    private void adjustColumns()
    {
        int width = 35;
        TableColumn column = table.getColumnModel().getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }

    private void initBuildings( boolean isAstro )
    {
        Object selection = buildings.getSelectedItem();
        selectedBuilding = selection;
        buildings.removeAllItems();
        Building[] b = ( isAstro ) ? Building.astroBuildings() : Building.planetBuildings();
        for( Building building : b )
        {
            buildings.addItem( new EnumWrapper<Building>( configData, building ) );
        }
        // restore selection
        if( selection != null )
        {
        	buildings.setSelectedItem( selection );
        }
        else
        {
        	buildings.setSelectedIndex( 0 );
        	selectedBuilding = null;
        }
        buildings.setMaximumRowCount( b.length );
    }
}
