/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.statistic;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.EnumWrapper;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Research;
import sitool.components.Stage;
import sitool.table.MultiLineTableCellHeaderRenderer;
import sitool.table.ResearchStatisticsTableModel;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class ResearchStatisticsPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -7866132208680948821L;

    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JComboBox<EnumWrapper<Research>> research = new JComboBox<>();
    private JSpinner researchCenter = new JSpinner();
    private JSpinner level = new JSpinner();
    private JLabel canvas = new JLabel();
    private JPanel center;
    private JTable table;
    private ResearchStatisticsTableModel model;

    private JLabel labelResearchLab = new JLabel();
    private JLabel labelResearch = new JLabel();
    private JLabel labelLevel = new JLabel();
    private JLabel labelMaxLevel = new JLabel();

    public ResearchStatisticsPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        canvas.setPreferredSize( new Dimension( 120, 120 ) );
        canvas.setHorizontalAlignment( SwingConstants.CENTER );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        researchCenter.setValue( findValue() );
        updateTable();
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        for( Research r : Research.values() )
        {
            research.addItem( new EnumWrapper<Research>( configData, r ) );
        }
        research.setSelectedIndex( 0 );
        research.setMaximumRowCount( Research.values().length );

        researchCenter.setModel( new SpinnerNumberModel( 1, 0, 150, 1 ) );
        researchCenter.setValue( findValue() );
        level.setModel( new SpinnerNumberModel( 35, 1, 150, 1 ) );

        JPanel west = new JPanel( new GridBagLayout() );
        center.add( west, BorderLayout.WEST );
        west.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridheight = 1;
        constraints.gridwidth = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.insets = new Insets( 2, 2, 2, 2 );

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 2;
        west.add( labelResearchLab, constraints );
        constraints.gridy = 1;
        constraints.gridwidth = 1;
        west.add( labelLevel, constraints );
        constraints.gridx = 1;
        west.add( researchCenter, constraints );
        constraints.gridx = 0;
        constraints.gridy = 2;
        west.add( labelResearch, constraints );
        constraints.gridx = 1;
        west.add( research, constraints );
        constraints.gridx = 0;
        constraints.gridy = 3;
        west.add( labelMaxLevel, constraints );
        constraints.gridx = 1;
        west.add( level, constraints );
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 2;
        constraints.weighty = 1.0;
        constraints.gridheight = GridBagConstraints.REMAINDER;
        west.add( canvas, constraints );

        model = new ResearchStatisticsTableModel( mainFrame );
        table = new TooltipTable( model );
        JTableHeader header = table.getTableHeader();
        TableCellRenderer renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table.setDefaultRenderer( Object.class, new ResourceTableCellRenderer() );

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        research.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                actionResearchChanged( e );
            }
        });
        level.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setMaxRows( level );
                adjustColumns();
            }

        });
        researchCenter.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setRecLabLevel( level );
                adjustColumns();
            }

        });

        return center;
    }

    private void updateTable()
    {
        labelResearchLab.setText( configData.getI18nText( "ResearchLab" ) );
        labelResearch.setText( configData.getI18nText( "Research" ) );
        labelLevel.setText( configData.getI18nText( "Level" ) );
        labelMaxLevel.setText( configData.getI18nText( "max.Level" ) );
        model.setRecLabLevel( ((Number)researchCenter.getValue()).intValue() );
        model.setMaxRows( ((Number)level.getValue()).intValue() );
        ItemEvent event;
        event = new ItemEvent( research, 0, research.getSelectedItem(), ItemEvent.SELECTED );
        actionResearchChanged( event );
    }

    @SuppressWarnings("unchecked")
    protected void actionResearchChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Research research = ( ( EnumWrapper<Research> )e.getItem()).getBase();
            model.setResearch( research );
            int l;

            // set some useful limits
            switch( research )
            {
                case blackhole:
                    l = 2;
                    break;
                case particle:
                    l = 5;
                    break;
                case teleportation:
                    l = 6;
                    break;
                case drive:
                    l = 8;
                    break;
                case graviton:
                case target:
                    l = 10;
                    break;
                default:
                    l = 25;
            }
            level.setValue( Integer.valueOf( l ) );

            canvas.setIcon( research.getIcon() );
            canvas.repaint();
            adjustColumns();
        }
    }

    private int findValue()
    {
        List<Planet> data = mainFrame.getConfigData().getMyPlanets();
        int rv = 1;
        for( Planet planet : data )
        {
            Stage stage = planet.getStage( Building.researchLab, false );
            if( stage != null && stage.getLevel() > rv )
            {
                rv = stage.getLevel();
            }
        }
        return rv;
    }

    private void adjustColumns()
    {
        int width = 35;
        TableColumn column = table.getColumnModel().getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }
}
