/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.statistic;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Stage;
import sitool.components.Version;
import sitool.table.MultiLineTableCellHeaderRenderer;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.ShipStatisticsTableModel;
import sitool.table.ShipStatisticsTableModel2;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class ShipStatisticsPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -8045686423181593884L;

    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JComboBox myPlanets = new JComboBox();
    private JCheckBox astro = new JCheckBox();
    private JCheckBox engineer = new JCheckBox();
    private JSpinner weaponsFactory = new JSpinner();
    private JSpinner msa = new JSpinner();
    private JPanel center;
    private JTable table;
    private JTable table2;
    private ShipStatisticsTableModel model;
    private ShipStatisticsTableModel2 model2;

    private JLabel planetLabel = new JLabel();
    private JLabel astroLabel = new JLabel();
    private JLabel weaponsLabel = new JLabel();
    private JLabel msaLabel = new JLabel();
    private JLabel engineerLabel = new JLabel();

    public ShipStatisticsPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateLabel();
        updateMyPlanets();
        updateTable();
        adjustColumns();
    }

    private void updateLabel()
    {
        planetLabel.setText( configData.getI18nText( "Planet" ) );
        astroLabel.setText( configData.getI18nText( "Asteroid" ) );
        weaponsLabel.setText( configData.getI18nText( "WeaponsFact.level", configData.getI18nText( Building.weaponsFactory.name() ) ) );
        msaLabel.setText( configData.getI18nText( "Msa.level" ) );
        engineerLabel.setText( configData.getI18nText( "fleet.engineer" ) );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        weaponsFactory.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );
        msa.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );

        JPanel west = new JPanel( new GridBagLayout() );
        center.add( west, BorderLayout.WEST );
        west.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.insets = new Insets( 2, 2, 2, 2 );

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( planetLabel, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astroLabel, constraints );
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( myPlanets, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astro, constraints );

        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( weaponsLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( weaponsFactory, constraints );
        constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( msaLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( msa, constraints );
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( engineerLabel, constraints );
        constraints.gridx = 2;
        constraints.gridwidth = 2;
        constraints.weightx = 0.0;
        west.add( engineer, constraints );
        constraints.gridx = 0;
        constraints.gridy = 5;
        constraints.gridwidth = 4;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.gridheight = GridBagConstraints.REMAINDER;
        west.add( new JLabel(), constraints );

        JPanel tablePanel = new JPanel();
        tablePanel.setLayout( new BoxLayout( tablePanel, BoxLayout.Y_AXIS) );
        center.add( tablePanel, BorderLayout.CENTER );

        model = new ShipStatisticsTableModel( mainFrame );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = -1843006337877960444L;

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = getPreferredSize();
                return size;
            }
        };
        JTableHeader header = table.getTableHeader();
        TableCellRenderer renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table.setDefaultRenderer( Object.class, new ResourceTableCellRenderer() );

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        tablePanel.add( scrollPane );

        model2 = new ShipStatisticsTableModel2( mainFrame );
        table2 = new TooltipTable( model2 )
        {
            private static final long serialVersionUID = -9091259936496789055L;

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = getPreferredSize();
                return size;
            }
        };

        header = table2.getTableHeader();
        renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table2.setDefaultRenderer( Object.class, new ResourceTableCellRenderer() );

        scrollPane = new JScrollPane( table2,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        tablePanel.add( scrollPane );

        myPlanets.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionPlanetChanged( e );
            }

        });
        weaponsFactory.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setWeaponsfactoryLevel( level );
                adjustColumns();
            }

        });
        msa.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setMsaLevel( level );
                adjustColumns();
            }

        });
        astro.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isAstro = ( e.getStateChange() == ItemEvent.SELECTED );
                Planet planet = ( Planet )myPlanets.getSelectedItem();
                int level;
                Stage stage;
                stage = planet.getStage( Building.weaponsFactory, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                weaponsFactory.setValue( level );
                stage = planet.getStage( Building.msa, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                msa.setValue( level );
            }
        });
        engineer.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isEngineer = ( e.getStateChange() == ItemEvent.SELECTED );
                model.setEngineer( isEngineer );
            }
        });

        return center;
    }

    private void updateTable()
    {
        model.fireTableStructureChanged();
        model2.fireTableStructureChanged();
        Version version = configData.getVersion();
        if( version != Version.v20 )
        {
            TableColumnModel columnModel = table2.getColumnModel();
            TableColumn column = columnModel.getColumn( ShipStatisticsTableModel2.ACCELERATION );
            columnModel.removeColumn( column );
            column = columnModel.getColumn( ShipStatisticsTableModel2.PREPTIME );
            columnModel.removeColumn( column );
        }
        ItemEvent event;
        event = new ItemEvent( myPlanets, 0, myPlanets.getSelectedItem(), ItemEvent.SELECTED );
        actionPlanetChanged( event );
    }

    private void updateMyPlanets()
    {
        Object selection = myPlanets.getSelectedItem();
        myPlanets.removeAllItems();
        myPlanets.addItem( new Planet( "-- -- --", new Position( 0, 0, 0) ) );
        List<Planet> ownPlanets = new ArrayList<Planet>( configData.getMyPlanets() );
        for( Planet planet : ownPlanets )
        {
            myPlanets.addItem( planet );
        }
        // restore selection
        if( selection != null )
        {
            myPlanets.setSelectedItem( selection );
        }
        else
        {
            myPlanets.setSelectedIndex( 0 );
        }
        myPlanets.setMaximumRowCount( myPlanets.getItemCount() );
    }

    protected void actionPlanetChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Planet planet = ( Planet )e.getItem();
            astro.setSelected( false );
            int level;
            Stage stage;
            stage = planet.getStage( Building.weaponsFactory, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            weaponsFactory.setValue( level );
            stage = planet.getStage( Building.msa, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            msa.setValue( level );
        }
    }

    private void adjustColumns()
    {
        int width = 135;
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column = columnModel.getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );

        columnModel = table2.getColumnModel();
        column = columnModel.getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }
}
