/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.List;

import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;

import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Defense;
import sitool.components.Fleet;
import sitool.components.Planet;

/**
 * @author graf
 *
 */
public class DefenseTableModel extends PlanetHeaderTableModel implements CellEditorListener
{
    private static final long serialVersionUID = 9045672684082762031L;

    public DefenseTableModel( MainFrame mainFrame, boolean isAsteroid )
    {
        super( mainFrame, isAsteroid );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int columns = mainFrame.getConfigData().getMyPlanets( !isAsteroid, isAsteroid ).size() + 1;
        return columns;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        String rv;
        if( columnIndex == 0 )
        {
            rv = "";
        }
        else
        {
            List<Planet> data = mainFrame.getConfigData().getMyPlanets( !isAsteroid, isAsteroid );
            Planet target = data.get( columnIndex - 1);
            //return a 2 line header, this is cheating, but, hey it works
            return formatHeader( target.getName(), target.getPosition().toString() );
        }
        return rv;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
     */
    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        if( columnIndex == 0 )
            return String.class;
        else
            return Number.class;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        // provide summary line
        return Defense.defenses( configData.getVersion() ).length + 1;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        List<Planet> data = mainFrame.getConfigData().getMyPlanets( !isAsteroid, isAsteroid );
        Object rv = null;
        int rows = Defense.defenses( configData.getVersion() ).length;

        if( columnIndex == 0 )
        {
            if( rowIndex == rows )
            {
                rv = getAllPoints();
            }
            else
            {
                Defense defense = Defense.defenses( configData.getVersion() )[rowIndex];
                rv = configData.getI18nText( defense.name() );
            }
        }
        else
        {
            Planet target = data.get( columnIndex - 1);
            if( rowIndex < rows )
            {
                Defense defense = Defense.defenses( configData.getVersion() )[rowIndex];
                long count = target.getFleet( isAsteroid ).getCount( defense );
                rv = Long.valueOf( count );
            }
            else if( rowIndex == rows )
            {
                rv = getPoints( columnIndex );
            }
        }
        return rv;
    }

    @Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
        boolean rv;
        rv = ( columnIndex > 0 && rowIndex < Defense.defenses( configData.getVersion() ).length );
		return rv;
	}


	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex)
	{
        List<Planet> data = mainFrame.getConfigData().getMyPlanets( !isAsteroid, isAsteroid );
        Planet target = data.get( columnIndex - 1 );

        Defense defense = Defense.defenses( configData.getVersion() )[rowIndex];

        Long count = Long.valueOf( value.toString() );
        target.getFleet( isAsteroid ).setCount( defense, count );
	}

    public void editingCanceled( ChangeEvent e )
    {
    }

    public void editingStopped( ChangeEvent e )
    {
        // update the status rows
        int rows = Defense.defenses( configData.getVersion() ).length;
        fireTableRowsUpdated( rows, rows );
    }

    protected Cost totalCost( Planet planet )
    {
        Cost total = new Cost( 0, 0, 0, 0 );
        Fleet fleet = planet.getFleet( isAsteroid );

        for( Defense defense : Defense.values() )
        {
            long count = fleet.getCount( defense );
            if( count > 0 )
            {
                Cost cost = new Cost( defense.getCost(), count );
                total.add( cost );
            }
        }
        return total;
    }

}
