/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

public class MultiLineTableCellHeaderRenderer implements TableCellRenderer
{
	private TableCellRenderer delegate;
	
	public MultiLineTableCellHeaderRenderer( TableCellRenderer delegate )
	{
		this.delegate = delegate;
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		Component renderer = delegate.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );
		if( renderer instanceof JLabel && value != null )
		{
			int alignment = ((JLabel)renderer).getHorizontalAlignment();
			StringBuilder sb = new StringBuilder();
			sb.append( value.toString() );
			boolean isHTML = sb.indexOf( "html" ) > 0;
			int start = sb.indexOf( "align" );
			if( isHTML && start > 0 )
			{
				start = sb.indexOf( "\"", start ) + 1;
				int stop = sb.indexOf( "\"", start );
				if( stop > start )
				{
					String token;
					switch( alignment )
					{
                        case SwingConstants.LEFT:
                        case SwingConstants.LEADING:
							token = "left";
							break;
                        case SwingConstants.RIGHT:
                        case SwingConstants.TRAILING:
							token = "right";
							break;
						case SwingConstants.CENTER:
						default:
							token = "center";
							break;
					}
					sb.replace( start, stop, token );
					value = sb.toString();
				}
			}
			renderer = delegate.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );
		}
		return renderer;
	}
}
