/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import sitool.MainFrame;
import sitool.components.Planet;
import sitool.fleet.DefenseBox;
import sitool.fleet.ShipBox;
import sitool.resources.BuildingBox;

public class PlanetTableHeaderMouseListener extends MouseAdapter
{
    private Type type;
    
    public PlanetTableHeaderMouseListener( Type type )
    {
        this.type = type;
    }

    public void mouseClicked(MouseEvent e)
    {
    	Object o = e.getSource();
    	if( o instanceof JTableHeader )
    	{
    		JTableHeader header = (JTableHeader) o;
    		JTable table = header.getTable();
    		TableModel tableModel = table.getModel();
    		if( tableModel instanceof PlanetHeaderTableModel )
    		{
    			PlanetHeaderTableModel model = (PlanetHeaderTableModel) tableModel;
                TableColumnModel columnModel = table.getColumnModel();
                int viewColumn = columnModel.getColumnIndexAtX(e.getX());
                int column = table.convertColumnIndexToModel(viewColumn);

                if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() >= 1 && column > 0 )
                {
	    			Planet planet = model.getPlanet( column );
	    			if( planet != null )
	    			{
    	    			MainFrame mainFrame = model.getMainFrame();
    	    	        JDialog dlg = getDialog( mainFrame, planet );
    	    	        mainFrame.showModalBox( dlg );
    	                header.repaint();
	    			}
                }
    		}
    	}
    }
    
    protected JDialog getDialog( MainFrame mainFrame, Planet planet )
    {
        JDialog rv = null;
        switch( type )
        {
            case Building:
                rv = new BuildingBox( mainFrame, planet, false );
                break;
            case Defense:
                rv = new DefenseBox( mainFrame, planet );
                break;
            case Ship:
                rv = new ShipBox( mainFrame, planet );
                break;
        }
        
        return rv;
    }
    
    static public enum Type
    {
        Building,
        Defense,
        Ship;
    }
    
}
