/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import sitool.components.Planet;
import sitool.components.Position;
import sitool.savecalc.SaveTarget;

/**
 * @author graf
 *
 */
public class SaveTargetComparator implements Comparator<SaveTarget>
{
    private SaveTargetComparatorMode mode;
    private Map<Planet, Integer> order = new HashMap<Planet, Integer>();

    public SaveTargetComparator( SaveTargetComparatorMode mode )
    {
        this.mode = mode;
    }

    public void setMode( SaveTargetComparatorMode mode )
    {
        this.mode = mode;
    }

    public void setOrder( List<Planet> myPlanets )
    {
        order.clear();
        for( int i = 0; i < myPlanets.size(); i++ )
        {
            order.put( myPlanets.get( i ), Integer.valueOf( i ) );
        }
    }

    /**
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare( SaveTarget o1, SaveTarget o2 )
    {
        int rv;
        switch( mode )
        {
            case SORT_BY_ORDER:
                rv = compareByOrder( o1, o2 );
                break;
            case SORT_BY_PLANET:
                rv = compareByPlanet( o1, o2 );
                break;
	    	case SORT_BY_VELOCITY:
	            rv = compareByVelocity( o1, o2 );
	            break;
            case SORT_BY_UNIT:
                rv = compareByUnit( o1, o2 );
                break;
            case SORT_BY_DELTA:
                rv = compareByDelta( o1, o2 );
                break;
            case SORT_BY_DISTANCE:
                rv = compareByDistance( o1, o2 );
                break;
            case SORT_BY_DURATION:
            default:
                rv = compareByDuration( o1, o2 );
                break;
        }
        return rv;
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByOrder( SaveTarget o1, SaveTarget o2 )
    {
        Integer i1 = order.get( o1.getOrigin() );
        Integer i2 = order.get( o2.getOrigin() );
        if( i1 == null && i2 == null )
        {
            return 0;
        }
        else if( i1 == null && i2 != null )
        {
            return 1;
        }
        else if( i1 != null && i2 == null )
        {
            return -1;
        }

        return i1.intValue() - i2.intValue();
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByPlanet( SaveTarget o1, SaveTarget o2 )
    {
    	int rv;
    	Position p1 = o1.getPlanet().getPosition();
    	Position p2 = o2.getPlanet().getPosition();
    	rv = p1.getGalaxy() - p2.getGalaxy();
    	if( rv == 0 )
    	{
        	rv = p1.getSystem() - p2.getSystem();
    	}
    	if( rv == 0 )
    	{
        	rv = p1.getPlanet() - p2.getPlanet();
    	}
        return rv;
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByVelocity( SaveTarget o1, SaveTarget o2 )
    {
        int rv = o1.getVelocity() - o2.getVelocity();

        if( rv == 0 )
        {
            rv = (int)(o1.getDuration() - o2.getDuration());
        }
        return rv;
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByUnit( SaveTarget o1, SaveTarget o2 )
    {
        int rv = o1.getUnit().compareTo( o2.getUnit() );

        if( rv == 0 )
        {
            rv = o1.getVelocity() - o2.getVelocity();
        }
        return rv;
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByDelta( SaveTarget o1, SaveTarget o2 )
    {
        long delta1 = o1.getDelta();
        long delta2 = o2.getDelta();

        int rv = (delta1 == delta2) ? 0 : (delta1 < delta2) ? -1 : 1;
        if( rv == 0 )
        {
            rv = o1.getVelocity() - o2.getVelocity();
        }
        return rv;
    }

    /**
     * @param o1
     * @param o2
     * @return
     */
    private int compareByDuration( SaveTarget o1, SaveTarget o2 )
    {
        int rv = (int)(o1.getDuration() - o2.getDuration());

        if( rv == 0 )
        {
            rv = o1.getVelocity() - o2.getVelocity();
        }
        return rv;
    }

    private int compareByDistance( SaveTarget o1, SaveTarget o2 )
    {
        return Boolean.compare(o1.isLongDistance(), o2.isLongDistance());
    }
}
