/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.table;

import sitool.MainFrame;
import sitool.components.Resource;
import sitool.components.Ship;
import sitool.components.Version;
import sitool.statistic.DefenseCalculator;

/**
 * @author graf
 *
 */
public class ShipStatisticsTableModel extends StatisticsTableModel
{
    private static final long serialVersionUID = -573269893088610821L;

    private int weaponsfactoryLevel = 0;
    private int msaLevel = 0;
    private boolean isEngineerEnabled = false;

    public ShipStatisticsTableModel( MainFrame mainFrame )
    {
        super( mainFrame );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = 6;

        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        String[] parts;
        switch( column )
        {
            case 0:
                rv = formatHeader( "" );
                break;
            case 1:
            case 2:
            case 3:
            case 4:
                parts = configData.getI18nText( Resource.values()[column-1].name() ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case 5:
                parts = configData.getI18nText( "Buildtime" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;

        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return Ship.ships( configData.getVersion() ).length;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        Ship ship = Ship.ships( configData.getVersion() )[rowIndex];
        switch( columnIndex )
        {
            case 0:
                rv = configData.getI18nText( ship.name() );
                break;
            case 1:
            case 2:
            case 3:
            case 4:
                rv = format( ship.getCost().getResource( Resource.values()[columnIndex-1] ) );
                break;
            case 5:
                Version version = configData.getVersion();
                rv = formatTime( DefenseCalculator.buildTime( ship.getCost(), isEngineerEnabled, version ) / configData.getSpeedFactor(Ship.class) );
                break;
        }
        return rv;
    }

    public boolean markResource( int row, int column )
    {
        return false;
    }

    public void setWeaponsfactoryLevel( int weaponsfactoryLevel )
    {
        this.weaponsfactoryLevel = weaponsfactoryLevel;
        fireTableDataChanged();
    }

    public void setMsaLevel( int msaLevel )
    {
        this.msaLevel = msaLevel;
        fireTableDataChanged();
    }

    public void setEngineer( boolean isEngineerEnabled )
    {
        this.isEngineerEnabled = isEngineerEnabled;
        fireTableDataChanged();
    }


    protected String format( long value )
    {
        String rv;
        if( value == 0 )
        {
            rv = " - ";
        }
        else
        {
            rv =  super.format( value );
        }
        return rv;
    }

    protected String formatTime( double buildTime )
    {
        buildTime /= ( (weaponsfactoryLevel > 0) ? weaponsfactoryLevel : 1 );
        buildTime /= Math.pow( 2, msaLevel );

        return super.formatTime( buildTime, RoundMillis.NONE );
    }
}
