/****************************************
 *  COPYRIGHT (C) 2022
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.AbstractCellEditor;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellEditor;

import sitool.ConfigData;
import sitool.components.Specialization;
import sitool.components.SpecializationBox;

/**
 * @author graf
 *
 */
public class SpecializationTableCellEditor extends AbstractCellEditor implements TableCellEditor {

    private static final long serialVersionUID = -4445507524364165070L;

    private final Specialization specialization;
    private final JLabel component = new JLabel();
    private final SpecializationBox specializationDialog;
    private final Component owner;

    public SpecializationTableCellEditor(ConfigData configData, Component owner) {
        specialization = new Specialization();
        this.owner = owner;
        component.setBorder( new EmptyBorder(1, 1, 1, 1) );
        specializationDialog = new SpecializationBox(configData, null, true);
        component.setHorizontalAlignment(SwingConstants.CENTER);
    }

    @Override
    public Specialization getCellEditorValue() {
        return specialization;
    }

    @Override
    public Component getTableCellEditorComponent( JTable table, Object value,
                    boolean isSelected, int row, int column ) {
        Specialization s = (value != null) ? (Specialization)value : new Specialization();
        specialization.setValue(s);
        component.setText(specialization.toString());
        final Point point = owner.getLocationOnScreen();
        final Rectangle rect = table.getCellRect(row, column, true);
        // rect is relative to the tables which is somewhat below the dialog
        point.translate(rect.x - rect.width, rect.y + 140 - rect.height / 2);
        // we need this function to continue immediately, show the dialog in the UI thread
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                specializationDialog.show(SpecializationTableCellEditor.this, specialization, point);
            }
        });
        return component;
    }

    @Override
    public boolean stopCellEditing() {
        specialization.setValue(specializationDialog.getSpecialization());
        return super.stopCellEditing();
    }
}
