/****************************************
 *  COPYRIGHT (C) 2009
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import sitool.components.Planet;
import sitool.components.Position;

/**
 * @author graf
 *
 */
public class TooltipTable extends JTable
{
	private static final long serialVersionUID = 5374293775888082770L;

    public TooltipTable()
    {
        super();
    }

    /**
     * @param numRows
     * @param numColumns
     */
    public TooltipTable( int numRows, int numColumns )
    {
        super( numRows, numColumns );
    }

    /**
     * @param rowData
     * @param columnNames
     */
    public TooltipTable( Object[][] rowData, Object[] columnNames )
    {
        super( rowData, columnNames );
    }

    /**
     * @param dm
     * @param cm
     * @param sm
     */
    public TooltipTable( TableModel dm, TableColumnModel cm, ListSelectionModel sm )
    {
        super( dm, cm, sm );
    }

    /**
     * @param dm
     * @param cm
     */
    public TooltipTable( TableModel dm, TableColumnModel cm )
    {
        super( dm, cm );
    }

    /**
     * @param dm
     */
    public TooltipTable( TableModel dm )
    {
        super( dm );
    }

    protected JTableHeader createDefaultTableHeader() 
    {
        return new TooltipTableHeader(columnModel);
    }

    @Override
	public String getToolTipText(MouseEvent event) 
    {
        String tooltip = null;
        int viewRow = rowAtPoint( event.getPoint() );
        int viewColumn = columnAtPoint( event.getPoint() );
        int modelRow = convertRowIndexToModel( viewRow );
        int modelColumn = convertColumnIndexToModel( viewColumn );
        if( modelRow >= 0 && modelColumn >= 0 )
        {
	        Object value = getModel().getValueAt( modelRow, modelColumn );
	        if( value instanceof String )
	        {
	        	tooltip = value.toString();
	        	if( tooltip.trim().length() == 0 )
	        	{
	        		tooltip = null;
	        	}
	        }
	        else if( value instanceof Number )
	        {
	        	if( ((Number)value).longValue() != 0 )
	        	{
	        		tooltip = value.toString();
	        	}
	        }
	        else if( value instanceof Planet || value instanceof Position )
	        {
	        	tooltip = value.toString();
	        }
        }
        return tooltip;
	}

}
