/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.technology;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.GradientLabel;
import sitool.components.ModulePane;
import sitool.components.Research;

/**
 * 
 * @author graf
 *
 */
public class ResearchTechTreePane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private ConfigData configData;
    private TechtreeDependency techtreeDependency;
    private JPanel center;
    
    private static final Research[] research = Research.values();
    private JLabel[] header = new JLabel[2];
    private JLabel[][] labels = new JLabel[2][research.length];
	
	public ResearchTechTreePane( MainFrame parent )
    {
        configData = parent.getConfigData();
        techtreeDependency = new TechtreeDependency( configData );
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        
        JPanel center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );
        
        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        header[0].setText( configData.getI18nText( "statistics.research" ) );
        header[1].setText( configData.getI18nText( "techtree.required" ) );
        for( int i = 0; i < research.length; i++ )
        {
            Color bg = ( techtreeDependency.isEnabled( research[i] ) ) ? TechTreePane.enabled : TechTreePane.disabled;
            labels[0][i].setText( configData.getI18nText( research[i].name() ) );
            labels[0][i].setBackground( bg );
            labels[1][i].setText( techtreeDependency.getDependencies( research[i] ) );
            labels[1][i].setBackground( bg );
        }
    }

    protected JPanel createCenterPanel()
    {
        center = new JPanel();
        center.setBorder( BorderFactory.createEmptyBorder( 4, 10, 4, 10 ) );
        center.setLayout( new BoxLayout( center, BoxLayout.Y_AXIS ) );
        
        JPanel panel = new JPanel( new GridLayout( 1, 2) );
        header[0] = new GradientLabel();
        panel.add( header[0] );
        header[1] = new GradientLabel();
        panel.add( header[1] );
        center.add( panel );
        for( int i = 0; i < research.length; i++ )
        {
            panel = new JPanel( new GridLayout( 1, 2) );
            labels[0][i] = new GradientLabel();
            panel.add( labels[0][i] );
            labels[1][i] = new GradientLabel();
            panel.add( labels[1][i] );
            center.add( panel );        
        }
        
        return center;
    }
    
}
