/****************************************
 *  COPYRIGHT (C) 2023
 *  Holger Graf
 ****************************************/
package sitool;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.fasterxml.jackson.jr.ob.JSON;
import com.fasterxml.jackson.jr.ob.JSON.Feature;

import sitool.components.Position;
import sitool.components.Version;
import sitool.json.PlanetVisibility;
import sitool.json.VisibilityMap;

/**
 *
 */
public class VisibilityImport {

    private static boolean parseExistence(String group) {
        int e = Integer.parseInt(group);
        return (e == 1);
    }

    private static Position parsePosition(String group) {
        int p = Integer.parseInt(group) - 1;
        int galaxy = p / 16 / 400 + 1;
        int system = (p / 16) % 400 + 1;
        int planet = p % 16 + 1;

        return new Position(galaxy, system, planet);
    }


    private static void initPlanetVisibility(Version version, String resName, Map<String, List<String>> planetVisibility) {
        Pattern pattern = Pattern.compile("(\\d+)=(\\d)");
        try (BufferedReader is = new BufferedReader(new FileReader(resName))) {
            String line;
            while((line = is.readLine()) != null) {
                Matcher m = pattern.matcher( line );
                if(m.matches()) {
                    Position position = parsePosition(m.group(1));
                    boolean exists = parseExistence(m.group(2));
                    String key = new Position(position.getGalaxy(), position.getSystem(), 0).toString();
                    if(position.getGalaxy() <= version.getUpperGalaxyBound()
                    && position.getSystem() <= version.getUpperSystemBound()) {
                        List<String> planets = planetVisibility.get(key);
                        if(planets == null) {
                            // store and reset
                            planets = new ArrayList<>();
                            planetVisibility.put(key, planets);
                        }
                        if(exists) {
                            planets.add(position.toString());
                        }
                    }
                }
            }
        }
        catch(IOException ex) {
            ex.printStackTrace();
        }
    }

    /**
     * @param args
     */
    public static void main( String[] args ) {
        VisibilityMap visibilityMap = new VisibilityMap();
        Map<Version,String> resources = new TreeMap<Version, String>() {
            private static final long serialVersionUID = 9093577383039296294L;
            {
                put(Version.sirius, "visibility/planet_visibility_sirius.txt");
                put(Version.genesis, "visibility/planet_visibility_genesis5.txt");
                put(Version.infinity, "visibility/planet_visibility_infinity.txt");
            }
        };
        for(Map.Entry<Version, String> entry : resources.entrySet()) {
            PlanetVisibility planetVisibility = new PlanetVisibility();
            visibilityMap.put(entry.getKey().name(), planetVisibility);
            initPlanetVisibility(entry.getKey(), entry.getValue(), planetVisibility);
        }
        try {
            JSON.std
            .with(Feature.PRETTY_PRINT_OUTPUT)
            .write(visibilityMap, new File("res/visibility.json"));
        }
        catch(IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

}
